@@ -2,7 +2,8 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.
+# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
 # the GNU General Public License version 2 as published by
@@ -33,10 +34,11 @@ from twisted.internet import error, defer, reactor
 from twisted.cred import error as crederror
 from twisted.python import rebuild, reflect
 
-from flumotion.common import bundle, common, errors, interfaces, log, keycards
-from flumotion.twisted import flavors
+from flumotion.common import common, errors, interfaces, log, pygobject
+from flumotion.common import keycards, worker, planet, medium, package
 # serializable worker and component state
-from flumotion.common import worker, planet
+from flumotion.twisted import flavors
+from flumotion.twisted.defer import defer_generator_method
 
 from flumotion.configure import configure
 from flumotion.common import reload
@@ -45,10 +47,104 @@ from flumotion.twisted import pb as fpb
 
 from flumotion.common.pygobject import gsignal, gproperty
 
-# FIXME: this is a Medium
+# FIXME: keycard should be created and handed to factory instead
+class AdminClientFactory(fpb.ReconnectingFPBClientFactory):
+    def __init__(self, medium, user, passwd):
+        """
+        @type medium:  AdminModel
+        @param user:   username to log in with
+        @param passwd: password to log in with
+        """
+        fpb.ReconnectingFPBClientFactory.__init__(self)
+
+        self.user = user
+        self.passwd = passwd
+        self.medium = medium
+        self.maxDelay = 20
+
+        self.hasBeenConnected = 0
+
+        # FIXME: try more than one auth method
+        #keycard = keycards.KeycardUACPP(user, passwd, 'localhost')
+        keycard = keycards.KeycardUACPCC(user, 'localhost')
+        # FIXME: decide on an admin name ?
+        keycard.avatarId = "admin"
+
+        # start logging in
+        self.startLogin(keycard, medium, interfaces.IAdminMedium)
+
+    def clientConnectionMade(self, broker):
+      self.hasBeenConnected = 1
+
+      fpb.ReconnectingFPBClientFactory.clientConnectionMade(self, broker)
+
+    def clientConnectionFailed(self, connector, reason):
+        """
+        @param reason: L{twisted.spread.pb.failure.Failure}
+        """
+        if reason.check(error.DNSLookupError):
+            self.debug('DNS lookup error')
+            self.medium.connectionFailed(reason)
+            return
+        elif reason.check(error.ConnectionRefusedError):
+            # If we're logging in for the first time, we want to make this a
+            # real error; we present a dialog, etc. 
+            # However, if we fail later on (e.g. manager shut down, and 
+            # hasn't yet been restarted), we want to keep trying to reconnect,
+            # so we just log a message.
+            self.debug("Connection refused error")
+            if not self.hasBeenConnected:
+                self.medium.connectionFailed(reason)
+                return
+
+        fpb.ReconnectingFPBClientFactory.clientConnectionFailed(self, 
+            connector, reason)
+        # delay is now updated
+        self.debug("will try reconnect in %f seconds" % self.delay)
+
+    # vmethod implementation
+    def gotDeferredLogin(self, d):
+        yield d
+
+        try:
+            result = d.value()
+            assert result
+            # if it's not a reference, we need to respond to a
+            # challenge...
+            if not isinstance(result, pb.RemoteReference):
+                keycard = result
+                keycard.setPassword(self.passwd)
+                self.log("_loginCallback: responding to challenge")
+                d = self.login(keycard, self.medium, interfaces.IAdminMedium)
+                yield d
+                result = d.value()
+
+            self.medium.setRemoteReference(result)
+
+        except errors.ConnectionFailedError:
+            self.debug("emitting connection-failed")
+            self.medium.emit('connection-failed', "I failed my master")
+            self.debug("emitted connection-failed")
+
+        except errors.ConnectionRefusedError:
+            self.debug("emitting connection-refused")
+            self.medium.emit('connection-refused')
+            self.debug("emitted connection-refused")
+
+        except crederror.UnauthorizedLogin:
+            # FIXME: unauthorized login emit !
+            self.debug("emitting connection-refused")
+            self.medium.emit('connection-refused')
+            self.debug("emitted connection-refused")
+
+        except Exception, e:
+            self.medium._defaultErrback(e)
+
+    gotDeferredLogin = defer_generator_method(gotDeferredLogin)
+        
 # FIXME: stop using signals, we can provide a richer interface with actual
 # objects and real interfaces for the views a model communicates with
-class AdminModel(pb.Referenceable, log.Loggable, gobject.GObject):
+class AdminModel(medium.BaseMedium, gobject.GObject):
     """
     I live in the admin client.
     I am a data model for any admin view implementing a UI to
@@ -59,8 +155,8 @@ class AdminModel(pb.Referenceable, log.Loggable, gobject.GObject):
     """
     gsignal('connected')
     gsignal('disconnected')
-    gsignal('connection-refused', str, int, bool)
-    gsignal('ui-state-changed', str, object)
+    gsignal('connection-refused')
+    gsignal('connection-failed', str)
     gsignal('component-property-changed', str, str, object)
     gsignal('reloading', str)
     gsignal('message', str)
@@ -70,67 +166,31 @@ class AdminModel(pb.Referenceable, log.Loggable, gobject.GObject):
 
     __implements__ = interfaces.IAdminMedium, flavors.IStateListener,
 
-    # appease pychecker
-    # FIXME: this is stupid, fix this properly ?
-    _components = {}
-    clientFactory = None
-    remote = None
-    _workerHeavenState = None
-    _views = []
-    _unbundler = None
-    state = 'disconnected'
-
-    user = passwd = host = port = use_insecure = None
+    # Public instance variables (read-only)
+    planet = None
 
     def __init__(self, username, password):
         self.__gobject_init__()
         
+        # All of these instance variables are private. Cuidado cabrones!
         self.user = username
         self.passwd = password
+        self.host = self.port = self.use_insecure = None
 
-        self.clientFactory = fpb.ReconnectingFPBClientFactory()
+        self.state = 'disconnected'
+        self.clientFactory = self._makeFactory(username, password)
         # 20 secs max for an admin to reconnect
         self.clientFactory.maxDelay = 20
 
-        self.remote = None
-
         self._components = {} # dict of components
-        self._planetState = None
+        self.planet = None
         self._workerHeavenState = None
         
         self._views = [] # all UI views I am serving
 
-        self._unbundler = bundle.Unbundler(configure.cachedir)
-
-        self.debug("logging in to ServerFactory")
-
-        # FIXME: one without address maybe ? or do we want manager to set it ?
-        # or do we set our guess and let manager correct ?
-        # FIXME: try both, one by one, and possibly others
-        #keycard = keycards.KeycardUACPP(username, password, 'localhost')
-        keycard = keycards.KeycardUACPCC(username, 'localhost')
-        # FIXME: decide on an admin name ?
-        keycard.avatarId = "admin"
- 
-        # start logging in
-        self.clientFactory.startLogin(keycard, self, interfaces.IAdminMedium)
-
-        # override gotDeferredLogin so we can add callbacks.
-        def gotDeferredLogin(d):
-            # add a callback to respond to the challenge
-            d.addCallback(self._loginCallback, password)
-            d.addErrback(self._accessDeniedErrback)
-            d.addErrback(self._connectionRefusedErrback)
-            # when the loginCallback finally succeeds, we are logged in
-            # and get a remote reference
-            d.addCallback(self.setRemoteReference)
-            d.addCallback(self._getPlanetState)
-            d.addCallback(self._getWorkerHeavenState)
-            d.addCallback(self._connectedCallback)
-            d.addErrback(self._defaultErrback)
-
-        # if this ever breaks, do real subclassing
-        self.clientFactory.gotDeferredLogin = gotDeferredLogin
+    # a method so mock testing frameworks can override it
+    def _makeFactory(self, username, password):
+        return AdminClientFactory(self, username, password)
 
     def connectToHost(self, host, port, use_insecure=False):
         'Connect to a host.'
@@ -147,98 +207,114 @@ class AdminModel(pb.Referenceable, log.Loggable, gobject.GObject):
             reactor.connectSSL(host, port, self.clientFactory,
                                ssl.ClientContextFactory())
 
-    ### our methods
-    def _loginCallback(self, result, password):
-        self.log("_loginCallback(result=%r, password=%s)" % (result, password))
-        assert result
-        # if we have a reference, we're in
-        if isinstance(result, pb.RemoteReference):
-            return result
-        # else, we need to respond to the challenge
-        keycard = result
-        keycard.setPassword(password)
-        self.log("_loginCallback: responding to challenge")
-        d = self.clientFactory.login(keycard, self, interfaces.IAdminMedium)
-        return d
-        
-    def _connectionRefusedErrback(self, failure):
-        failure.trap(error.ConnectionRefusedError)
-        self.debug("emitting connection-refused")
-        self.emit('connection-refused', self.host, self.port, self.use_insecure)
-        self.debug("emitted connection-refused")
+        def connected(model, d, ids):
+            map(model.disconnect, ids)
+            d.callback(model)
 
-    def _accessDeniedErrback(self, failure):
-        failure.trap(crederror.UnauthorizedLogin)
-        # FIXME: unauthorized login emit !
-        self.debug("emitting connection-refused")
-        self.emit('connection-refused', self.host, self.port, self.use_insecure)
-        self.debug("emitted connection-refused")
+        def connection_refused(model, d, ids):
+            map(model.disconnect, ids)
+            d.errback(errors.ConnectionRefusedError())
 
-    def _getPlanetState(self, result):
-        # called after logging in and getting a reference
-        d = self.callRemote('getPlanetState')
-        d.addCallback(lambda result: self._setPlanetState(result))
-        d.addCallback(lambda result, s: s.debug('got planet state'), self)
+        def connection_failed(model, reason, d, ids):
+            map(model.disconnect, ids)
+            d.errback(errors.ConnectionFailedError(reason))
 
-        # if you don't return the deferred, they don't chain properly !       
+        d = defer.Deferred()
+        ids = []
+        ids.append(self.connect('connected', connected, d, ids))
+        ids.append(self.connect('connection-refused',
+                                connection_refused, d, ids))
+        ids.append(self.connect('connection-failed',
+                                connection_failed, d, ids))
         return d
 
-    def _getWorkerHeavenState(self, result):
-        # called after logging in and getting a reference
-        d = self.callRemote('getWorkerHeavenState')
-        d.addCallback(lambda result: self._setWorkerHeavenState(result))
-        d.addCallback(lambda result, s: s.debug('got worker state'), self)
-        return d
-        
-    def _writeConnection(self):
-        s = ''.join(['<connection>',
-                     '<host>%s</host>' % self.host,
-                     '<manager>%s</manager>' % self._planetState.get('name'),
-                     '<port>%d</port>' % self.port,
-                     '<use_insecure>%d</use_insecure>' 
-                     % (self.use_insecure and 1 or 0),
-                     '<user>%s</user>' % self.user,
-                     '<passwd>%s</passwd>' % self.passwd,
-                     '</connection>'])
-        
-        sum = md5.new(s).hexdigest()
-        f = os.path.join(configure.registrydir, '%s.connection' % sum)
-        h = open(f, 'w')
-        h.write(s)
-        h.close()
-
-    def _connectedCallback(self, result):
-        self.debug('Connected to manager and retrieved all state')
-        self.state = 'connected'
-        self._writeConnection()
-        self.emit('connected')
-        
     # default Errback
+    # FIXME: we can set it up with a list of types not to warn for ?
     def _defaultErrback(self, failure):
-        self.debug('Unhandled deferred failure: %r (%s)' % (
-            failure.type, failure.getErrorMessage()))
+        self.debug('Possibly unhandled deferred failure: %r (%s)' % (
+            failure, failure.getErrorMessage()))
         return failure
 
     def reconnect(self):
         self.debug('asked to log in again')
+        self.clientFactory.stopTrying()
+        # this also makes it try to connect again
         self.clientFactory.resetDelay()
-        #self.clientFactory.retry(self.clientFactory.connector)
+        self.connectToHost(self.host, self.port, self.use_insecure)
+
+    # FIXME: give these three sensible names
+    def adminInfoStr(self):
+        return '%s@%s:%s' % (self.user, self.host, self.port)
+
+    def connectionInfoStr(self):
+        return '%s:%s (%s)' % (self.host, self.port,
+                               self.use_insecure and 'http' or 'https')
+
+    # used in fgc
+    def managerInfoStr(self):
+        assert self.planet
+        return '%s (%s:%s)' % (self.planet.get('name'), self.host, self.port)
+
+    def connectionFailed(self, reason):
+        # called by client factory
+        if reason.check(error.DNSLookupError):
+            self.debug('emitting connection-failed')
+            self.emit('connection-failed', "Could not look up host '%s'." %
+                self.host)
+            self.debug('emited connection-failed')
+        if reason.check(error.ConnectionRefusedError):
+            self.debug('emitting connection-failed')
+            self.emit('connection-failed', "Could not connect to host '%s' on port %d." %
+                (self.host, self.port))
+            self.debug('emited connection-failed')
 
-    ### IMedium methods
     def setRemoteReference(self, remoteReference):
-        # we get called when we managed to log in to a manager
-        self.debug("setRemoteReference: %s" % remoteReference)
-        self.remote = remoteReference
-        self.remote.notifyOnDisconnect(self._remoteDisconnected)
+        def writeConnection():
+            s = ''.join(['<connection>',
+                         '<host>%s</host>' % self.host,
+                         '<manager>%s</manager>' % self.planet.get('name'),
+                         '<port>%d</port>' % self.port,
+                         '<use_insecure>%d</use_insecure>' 
+                         % (self.use_insecure and 1 or 0),
+                         '<user>%s</user>' % self.user,
+                         '<passwd>%s</passwd>' % self.passwd,
+                         '</connection>'])
+            
+            sum = md5.new(s).hexdigest()
+            f = os.path.join(configure.registrydir, '%s.connection' % sum)
+            h = open(f, 'w')
+            h.write(s)
+            h.close()
 
-    def _remoteDisconnected(self, remoteReference):
-        self.debug("emitting disconnected")
-        self.state = 'disconnected'
-        self.emit('disconnected')
-        self.debug("emitted disconnected")
+        # chain up
+        medium.BaseMedium.setRemoteReference(self, remoteReference)
 
-    def hasRemoteReference(self):
-        return self.remote != None
+        # fixme: push the disconnect notification upstream
+        def remoteDisconnected(remoteReference):
+            self.debug("emitting disconnected")
+            self.state = 'disconnected'
+            self.emit('disconnected')
+            self.debug("emitted disconnected")
+        self.remote.notifyOnDisconnect(remoteDisconnected)
+
+        d = self.callRemote('getPlanetState')
+        yield d
+        self.planet = d.value()
+        # monkey, Monkey, MONKEYPATCH!!!!!
+        self.planet.admin = self
+        self.debug('got planet state')
+        self.callViews('setPlanetState', self.planet)
+
+        d = self.callRemote('getWorkerHeavenState')
+        yield d
+        self._workerHeavenState = d.value()
+        self.debug('got worker state')
+        
+        writeConnection()
+        self.debug('Connected to manager and retrieved all state')
+        self.state = 'connected'
+        self.emit('connected')
+    setRemoteReference = defer_generator_method(setRemoteReference)
 
     def callViews(self, methodName, *args, **kwargs):
         """
@@ -299,14 +375,6 @@ class AdminModel(pb.Referenceable, log.Loggable, gobject.GObject):
             self._views.remove(view)
         
     ## generic remote call methods
-    def callRemote(self, methodName, *args, **kwargs):
-        """
-        Call the given remote method on the manager-side AdminAvatar.
-        """
-        if not self.remote:
-            raise errors.ManagerNotConnectedError
-        return self.remote.callRemote(methodName, *args, **kwargs)
-
     def componentCallRemote(self, componentState, methodName, *args, **kwargs):
         """
         Call the given method on the given component with the given args.
@@ -402,6 +470,31 @@ class AdminModel(pb.Referenceable, log.Loggable, gobject.GObject):
             d.addErrback(self._defaultErrback)
         return d
 
+    def reload_async(self, duckport):
+        name = reflect.filenameToModuleName(__file__)
+
+        self.info("rebuilding '%s'" % name)
+        rebuild.rebuild(sys.modules[name])
+
+        d = self.reloadManager()
+        yield d
+        try:
+            d.value()
+            duckport.write('Reloaded manager')
+        except Exception, e:
+            duckport.write('Failed to reload manager: %s' % e)
+
+        for name in self._components.keys():
+            d = self.reloadComponent(name)
+            yield d
+            try:
+                d.value()
+                duckport.write('Reloaded component %s' % name)
+            except Exception, e:
+                duckport.write('Failed to reload component %s: %s' % (name, e))
+        duckport.close()
+    reload_async = defer_generator_method(reload_async)
+
     def reloadManager(self):
         """
         Tell the manager to reload its code.
@@ -413,7 +506,7 @@ class AdminModel(pb.Referenceable, log.Loggable, gobject.GObject):
 
         self.emit('reloading', 'manager')
         self.info("reloading manager code")
-        d = self.remote.callRemote('reloadManager')
+        d = self.callRemote('reloadManager')
         d.addCallback(_reloaded, self)
         d.addErrback(self._defaultErrback)
         return d
@@ -432,23 +525,25 @@ class AdminModel(pb.Referenceable, log.Loggable, gobject.GObject):
         name = componentState.get('name')
         self.info("reloading component %s code" % name)
         self.emit('reloading', name)
-        d = self.remote.callRemote('reloadComponent', componentState)
+        d = self.callRemote('reloadComponent', componentState)
         d.addCallback(_reloaded, self, componentState)
         d.addErrback(self._defaultErrback)
         return d
 
     ## manager remote methods
     def loadConfiguration(self, xml_string):
-        return self.remote.callRemote('loadConfiguration', xml_string)
+        return self.callRemote('loadConfiguration', xml_string)
 
     def getConfiguration(self):
-        return self.remote.callRemote('getConfiguration')
+        return self.callRemote('getConfiguration')
 
     def cleanComponents(self):
-        return self.remote.callRemote('cleanComponents')
+        return self.callRemote('cleanComponents')
 
     # function to get remote code for admin parts
     # FIXME: rename slightly ?
+    # FIXME: still have hard-coded os.path.join stuff in here for md5sum,
+    # move to bundleloader ?
     def getEntry(self, componentState, type):
         """
         Do everything needed to set up the entry point for the given
@@ -456,157 +551,24 @@ class AdminModel(pb.Referenceable, log.Loggable, gobject.GObject):
 
         Caller is responsible for adding errbacks to the deferred.
 
-        Returns: a deferred returning (entryPath, filename, methodName)
+        Returns: a deferred returning (entryPath, filename, methodName) with
+                 entryPath: the full local path to the bundle's base
+                 fileName:  the relative location of the bundled file
+                methodName: the method to instantiate with
         """
-        
-        def _getEntryCallback(result, componentState, type):
-            # callback for getting the entry.  Will request bundle sums
-            # based on filename given to me
-            self.debug('_getEntryCallback: result %r' % (result, ))
-
-            filename, methodName = result
-            self.debug("entry point for %r of type %s is in file %s and method %s" % (componentState, type, filename, methodName))
-            # request bundle sums
-            d = self.callRemote('getBundleSumsByFile', filename)
-            d.addCallback(_getBundleSumsCallback, filename, methodName)
-            d.addErrback(self._defaultErrback)
-            return d
-
-        def _getBundleSumsCallback(result, filename, methodName):
-            # callback receiving bundle sums.  Will remote call to get
-            # all missing zip files
-            sums = result # ordered from highest to lowest dependency
-            entryName, entrySum = sums[0]
-            self.debug('_getBundleSumsCallback: %d sums received' % len(sums))
-
-            # get path where entry bundle is stored
-            entryPath = self._unbundler.unbundlePathByInfo(entryName, entrySum)
-
-            # check which zips to request from manager
-            cachedPaths = [] # list of cached paths to register later
-            missing = [] # list of missing bundles
-
-            for name, sum in sums:
-                dir = os.path.join(configure.cachedir, name, sum)
-                if not os.path.exists(dir):
-                    missing.append(name)
-                else:
-                    cachedPaths.append(dir)
-
-            cachedPaths.reverse()
-            missing.reverse()
-
-            self.debug('_getBundleSumsCallback: %d bundles missing' %
-                len(missing))
-            if missing:
-                self.debug('_getBundleSumsCallback: requesting zips %r' %
-                    missing)
-                d = self.callRemote('getBundleZips', missing)
-                d.addCallback(_getBundleZipsCallback, entryPath, missing,
-                    cachedPaths, filename, methodName)
-                d.addErrback(self._defaultErrback)
-                return d
-            else:
-                retval = (entryPath, filename, methodName)
-                self.debug('_getBundleSumsCallback: returning %r' % (
-                    retval, ))
-                self._registerCachedPaths(cachedPaths)
-                return retval
-
-        def _getBundleZipsCallback(result, entryPath, missing, cachedPaths,
-            filename, methodName):
-            # callback to receive zips.  Will set up zips, register package
-            # paths and finally
-            # return physical location of entry file and method
-            self.debug('_getBundleZipsCallback: received %d zips' % len(result))
-            # we use missing because that way we get the
-            # list of dependencies from lowest to highest and register
-            # package paths in the correct order; since we need to
-            # register package paths one by one for the namedAny's
-            # FIXME: missing can contain duplicate entries, remove ?
-            for name in missing:
-                if name not in result.keys():
-                    msg = "Missing bundle %s was not received" % name
-                    self.warning(msg)
-                    raise errors.NoBundleError(msg)
-
-                zip = result[name]
-                b = bundle.Bundle(name)
-                b.setZip(zip)
-                dir = self._unbundler.unbundle(b)
-                self.debug("unpacked bundle %s to dir %s" % (name, dir))
-                self.debug("unpacked bundle %s, registering PackagePath %s" % (
-                    dir, name))
-                common.registerPackagePath(dir)
-
-            # now make sure all cachedPaths are registered
-            # FIXME: does it matter we already did some before ?
-            self._registerCachedPaths(cachedPaths)
-
-            retval = (entryPath, filename, methodName)
-            self.debug('_getBundleSumsCallback: returning %r' % (
-                retval, ))
-            return retval
-
-        # start chain
         d = self.callRemote('getEntryByType', componentState, type)
-        d.addCallback(_getEntryCallback, componentState, type)
-        # our caller should handle errbacks
-        # d.addErrback(self._defaultErrback)
-        return d
+        yield d
 
-    def _registerCachedPaths(self, paths):
-        for dir in paths:
-            self.debug("registering cached PackagePath %s" % dir)
-            common.registerPackagePath(dir)
+        fileName, methodName = d.value()
+            
+        self.debug("entry for %r of type %s is in file %s and method %s" % (
+            componentState, type, fileName, methodName))
+        d = self.bundleLoader.getBundles(fileName=fileName)
+        yield d
 
-    def getBundledFile(self, bundledPath):
-        """
-        Do everything needed to get the given bundled file.
-
-        Returns: a deferred returning the absolute path to a local copy
-        of the given file.
-        """
-        def _getBundleSumsCallback(result, bundledPath):
-            # callback receiving bundle sums.  Will remote call to get
-            # the zip if it's missing
-            sums = result
-            bundleName, bundleSum = sums[0]
-            self.debug('bundledPath %s is in bundle %s' % (bundledPath,
-                bundleName))
-
-            # get path where this bundle is cached
-            cachePath = self._unbundler.unbundlePathByInfo(bundleName,
-                bundleSum)
-
-            if not os.path.exists(cachePath):
-                self.debug('_getBundleSumsCallback: requesting zip %s' %
-                    bundleName)
-                d = self.callRemote('getBundleZips', (bundleName, ))
-                d.addCallback(_getBundleZipsCallback, bundleName, bundledPath)
-                d.addErrback(self._defaultErrback)
-                self.debug("found %s in dir %s" % (bundleName, cachePath))
-                return d
-            else:
-                return os.path.join(cachePath, bundledPath)
-
-        def _getBundleZipsCallback(result, bundleName, bundledPath):
-            # callback to receive zip.  Will set up zips and finally
-            # return physical location of bundledPath
-            self.debug('_getBundleZipsCallback: received %d zips' % len(result))
-            zip = result[bundleName]
-            b = bundle.Bundle()
-            b.setZip(zip)
-            dir = self._unbundler.unbundle(b)
-            self.debug("unpacked %s to dir %s" % (bundleName, dir))
-
-            return os.path.join(dir, bundledPath)
-
-        # start chain
-        d = self.callRemote('getBundleSumsByFile', bundledPath)
-        d.addCallback(_getBundleSumsCallback, bundledPath)
-        d.addErrback(self._defaultErrback)
-        return d
+        name, bundlePath = d.value()[-1]
+        yield (bundlePath, fileName, methodName)
+    getEntry = defer_generator_method(getEntry)
 
     ## worker remote methods
     def checkElements(self, workerName, elements):
@@ -620,9 +582,10 @@ class AdminModel(pb.Referenceable, log.Loggable, gobject.GObject):
         worker does not already have the module, or it is out of date,
         it will be retrieved from the manager.
 
-        @rtype: L{twisted.internet.defer.Deferred}
+        @rtype: L{twisted.internet.defer.Deferred} firing an
+                L{flumotion.common.messages.Result}
         """
-        return self.workerCallRemote(workerName, 'runProc', moduleName,
+        return self.workerCallRemote(workerName, 'runFunction', moduleName,
                                      functionName, *args, **kwargs)
     
     # FIXME: this should not be allowed to be called, move away
@@ -632,18 +595,10 @@ class AdminModel(pb.Referenceable, log.Loggable, gobject.GObject):
         return self._components
     getComponents = get_components
     
-    def _setPlanetState(self, planetState):
-        self.debug('setting planetState %r' % planetState)
-        self._planetState = planetState
-        self.callViews('setPlanetState', planetState)
-
-    def getPlanetState(self):
-        return self._planetState
-           
     def _setWorkerHeavenState(self, state):
         self._workerHeavenState = state
 
     def getWorkerHeavenState(self):
         return self._workerHeavenState
 
-gobject.type_register(AdminModel)
+pygobject.type_register(AdminModel)
