@@ -2,7 +2,8 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.
+# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
 # the GNU General Public License version 2 as published by
@@ -19,23 +20,28 @@
 # Headers in this file shall remain intact.
 
 import os
-import md5
+import os.path
 import sys
 
+from gettext import gettext as _
+
 import gobject
 from gtk import gdk
 import gtk
 import gtk.glade
 
-from twisted.internet import reactor
+from twisted.internet import reactor, defer
 from twisted.python import rebuild
 
 from flumotion.admin.admin import AdminModel
-from flumotion.admin.gtk import dialogs, parts, connections
+from flumotion.admin import connections
+from flumotion.admin.gtk import dialogs, parts, message
+from flumotion.admin.gtk import connections as gtkconnections
 from flumotion.configure import configure
-from flumotion.common import errors, log, worker, planet, common
+from flumotion.common import errors, log, worker, planet, common, pygobject
 from flumotion.manager import admin # Register types
 from flumotion.twisted import flavors, reflect
+from flumotion.ui import icons, trayicon
 
 from flumotion.common.planet import moods
 from flumotion.common.pygobject import gsignal
@@ -58,6 +64,8 @@ class Window(log.Loggable, gobject.GObject):
 
         self.widgets = {}
         self.debug('creating UI')
+        self._trayicon = None
+
         self._create_ui()
 
         self._append_recent_connections()
@@ -93,7 +101,8 @@ class Window(log.Loggable, gobject.GObject):
         self.admin.connect('disconnected', self.admin_disconnected_cb)
         self.admin.connect('connection-refused',
                            self.admin_connection_refused_cb)
-        self.admin.connect('ui-state-changed', self.admin_ui_state_changed_cb)
+        self.admin.connect('connection-failed',
+                           self.admin_connection_failed_cb)
         self.admin.connect('component-property-changed',
             self.property_changed_cb)
         self.admin.connect('update', self.admin_update_cb)
@@ -118,33 +127,31 @@ class Window(log.Loggable, gobject.GObject):
         widgets = self.widgets
 
         window = self.window = widgets['main_window']
-        iconfile = os.path.join(configure.imagedir, 'fluendo.png')
-        gtk.window_set_default_icon_from_file(iconfile)
-        window.set_icon_from_file(iconfile)
         
         def set_icon(proc, size, name):
-            f = os.path.join(configure.imagedir, '%dx%d' % (size,size), name)
             i = gtk.Image()
-            i.set_from_file(f)
+            i.set_from_stock('flumotion-'+name, size)
             proc(i)
+            i.show()
         
         def make_menu_proc(m): # $%^& pychecker!
             return lambda f: m.set_property('image', f)
         def menu_set_icon(m, name):
-            set_icon(make_menu_proc(m), 16, name)
+            set_icon(make_menu_proc(m), gtk.ICON_SIZE_MENU, name)
+            m.show()
         
         def tool_set_icon(m, name):
-            set_icon(m.set_icon_widget, 24, name)
+            set_icon(m.set_icon_widget, gtk.ICON_SIZE_SMALL_TOOLBAR, name)
 
-        iconfile = os.path.join(configure.imagedir, 'fluendo.png')
-        gtk.window_set_default_icon_from_file(iconfile)
+        menu_set_icon(widgets['menuitem_manage_run_wizard'], 'wizard')
+        tool_set_icon(widgets['toolbutton_wizard'], 'wizard')
+        menu_set_icon(widgets['menuitem_manage_start_component'], 'play')
+        tool_set_icon(widgets['toolbutton_start_component'], 'play')
+        menu_set_icon(widgets['menuitem_manage_stop_component'], 'pause')
+        tool_set_icon(widgets['toolbutton_stop_component'], 'pause')
 
-        menu_set_icon(widgets['menuitem_manage_run_wizard'], 'wizard.png')
-        tool_set_icon(widgets['toolbutton_wizard'], 'wizard.png')
-        menu_set_icon(widgets['menuitem_manage_start_component'], 'play.png')
-        tool_set_icon(widgets['toolbutton_start_component'], 'play.png')
-        menu_set_icon(widgets['menuitem_manage_stop_component'], 'pause.png')
-        tool_set_icon(widgets['toolbutton_stop_component'], 'pause.png')
+        self._trayicon = trayicon.FluTrayIcon(self)
+        self._trayicon.set_tooltip(_('Not connected'))
 
         self.hpaned = widgets['hpaned']
  
@@ -162,30 +169,31 @@ class Window(log.Loggable, gobject.GObject):
         self.components_view.connect('notify::can-stop-any',
                                      self.start_stop_notify_cb)
         self.start_stop_notify_cb()
+
+        self._messages_view = widgets['messages_view']
+        self._messages_view.hide()
+
         return window
 
-    def open_connected_cb(self, model, ids):
-        map(model.disconnect, ids)
-        self.window.set_sensitive(True)
-        self._setAdminModel(model)
-        self._append_recent_connections()
-
-    def open_refused_cb(self, model, host, port, use_insecure, ids):
-        map(model.disconnect, ids)
-        self.window.set_sensitive(True)
-        print '\n\nconnection refused, try again'
-        print 'FIXME: make a proper errbox'
-
     def on_open_connection(self, config):
         model = AdminModel(config['user'], config['passwd'])
-        model.connectToHost(config['host'], config['port'],
-                            config['use_insecure'])
+        d = model.connectToHost(config['host'], config['port'],
+                                config['use_insecure'])
+        self._trayicon.set_tooltip(_("Connecting to %s:%s") %
+            (config['host'], config['port']))
 
-        ids = []
-        ids.append(model.connect('connected',
-                                 self.open_connected_cb, ids))
-        ids.append(model.connect('connection-refused',
-                                 self.open_refused_cb, ids))
+        def connected(model):
+            self.window.set_sensitive(True)
+            self._setAdminModel(model)
+            self._append_recent_connections()
+
+        def refused(failure):
+            failure.trap(errors.ConnectionRefusedError)
+            dialogs.connection_refused_modal_message(config['host'],
+                                                     self.window)
+            self.window.set_sensitive(True)
+
+        d.addCallback(connected).addErrback(refused)
         self.window.set_sensitive(False)
 
     def on_recent_activate(self, widget, state):
@@ -203,7 +211,7 @@ class Window(log.Loggable, gobject.GObject):
             else:
                 menu.remove(w)
 
-        clist = connections.Connections().get_recent_connections()
+        clist = connections.get_recent_connections()
         if not clist:
             return
 
@@ -211,8 +219,8 @@ class Window(log.Loggable, gobject.GObject):
             i.show()
             gtk.MenuShell.append(menu, i) # $%^&* pychecker
         def append_txt(c, n):
-            i = gtk.MenuItem(c[0])
-            i.connect('activate', self.on_recent_activate, c[1])
+            i = gtk.MenuItem(c['name'])
+            i.connect('activate', self.on_recent_activate, c['state'])
             append(i)
             
         append(gtk.SeparatorMenuItem())
@@ -247,16 +255,8 @@ class Window(log.Loggable, gobject.GObject):
         # methodName has historically been GUIClass
         instance = None
 
-        # if there's a current component being shown, give it a chance
-        # to clean up
-        if self.current_component:
-            if hasattr(self.current_component, 'cleanup'):
-                self.debug('Cleaning up current component view')
-                self.current_component.cleanup()
-        self.current_component = None
-
         name = state.get('name')
-        self.statusbar.set('main', "Loading UI for %s ..." % name)
+        self.statusbar.set('main', _("Loading UI for %s ...") % name)
 
         moduleName = common.pathToModuleName(fileName)
         statement = 'import %s' % moduleName
@@ -272,9 +272,9 @@ class Window(log.Loggable, gobject.GObject):
             self.warning(msg)
             raise errors.EntrySyntaxError(msg)
         except NameError, e:
-            # the syntax error can happen in the entry file, or any import
-            msg = "NameError while executing %s: %s" % (fileName,
-                " ".join(e.args))
+            msg = "NameError at while executing %s: %s" % (
+                fileName, " ".join(e.args))
+            raise
             self.warning(msg)
             raise errors.EntrySyntaxError(msg)
         except ImportError, e:
@@ -294,27 +294,38 @@ class Window(log.Loggable, gobject.GObject):
             raise #FIXME: something appropriate
         klass = getattr(module, methodName)
 
-        # instantiate the GUIClass, giving ourself as the first argument
-        # FIXME: we cheat by giving the view as second for now,
-        # but let's decide for either view or model
-        instance = klass(state, self.admin, self)
+        # instantiate the GUIClass
+        instance = klass(state, self.admin)
         self.debug("Created entry instance %r" % instance)
-        instance.setup()
-        nodes = instance.getNodes()
+        d = instance.setup()
+        # F0.2: setup should return a deferred
+        if not d:
+            d = defer.succeed(None)
+        d.addCallback(self._setupCallback, name, instance)
+
+    def _setupCallback(self, result, name, instance):
         notebook = gtk.Notebook()
         nodeWidgets = {}
+        nodes = instance.getNodes()
+
+        # F0.2
+        for nodeName, node in nodes.items():
+            if not node.title:
+                self.warning('Node %s does not have a title, please fix' %
+                    nodeName)
+                node.title = nodeName
 
         self.statusbar.clear('main')
         # create pages for all nodes, and just show a loading label for
         # now
-        for nodeName in nodes.keys():
-            self.debug("Creating node for %s" % nodeName)
-            label = gtk.Label('Loading UI for %s ...' % nodeName)
+        for node in nodes.values():
+            self.debug("Creating node for %s" % node.title)
+            label = gtk.Label(_('Loading UI for %s ...') % node.title)
             table = gtk.Table(1, 1)
             table.add(label)
-            nodeWidgets[nodeName] = table
+            nodeWidgets[node.title] = table
 
-            notebook.append_page(table, gtk.Label(nodeName))
+            notebook.append_page(table, gtk.Label(node.title))
             
         # put "loading" widget in
         old = self.hpaned.get_child2()
@@ -325,14 +336,14 @@ class Window(log.Loggable, gobject.GObject):
         # trigger node rendering
         # FIXME: might be better to do these one by one, in order,
         # so the status bar can show what happens
-        for nodeName in nodes.keys():
+        for node in nodes.values():
             mid = self.statusbar.push('notebook',
-                "Loading tab %s for %s ..." % (nodeName, name))
-            node = nodes[nodeName]
+                _("Loading tab %s for %s ...") % (node.title, name))
+            node.statusbar = self.statusbar # hack
             d = node.render()
-            d.addCallback(self._nodeRenderCallback, nodeName,
+            d.addCallback(self._nodeRenderCallback, node.title,
                 instance, nodeWidgets, mid)
-            # FIXME: errback
+            d.addErrback(self._nodeRenderErrback, node.title)
 
     def _nodeRenderCallback(self, widget, nodeName, gtkAdminInstance,
         nodeWidgets, mid):
@@ -346,7 +357,7 @@ class Window(log.Loggable, gobject.GObject):
         
         if not widget:
             self.warning(".render() did not return an object")
-            widget = gtk.Label('%s does not have a UI yet' % nodeName)
+            widget = gtk.Label(_('%s does not have a UI yet') % nodeName)
         else:
             parent = widget.get_parent()
             if parent:
@@ -357,6 +368,11 @@ class Window(log.Loggable, gobject.GObject):
 
         self.current_component = gtkAdminInstance
 
+    def _nodeRenderErrback(self, failure, nodeName):
+        self.debug('Could not render node %s: %r, %s' % (nodeName,
+            failure, failure.getErrorMessage()))
+        self.warning('Could not render node %s' % nodeName)
+
     ### IAdminView interface methods: FIXME: create interface somewhere
     ## Confusingly enough, this procedure is called by remote objects to
     ## operate on the client ui. I think. It is *not* for calling
@@ -472,6 +488,10 @@ class Window(log.Loggable, gobject.GObject):
             self.statusbar.set('main', value)
         elif key == 'mood':
             self._set_stop_start_component_sensitive()
+            current = self.components_view.get_selected_name()
+            if value == moods.sleeping.value:
+                if state.get('name') == current:
+                    self._messages_view.clear()
 
     def stateAppend(self, state, key, value):
         if isinstance(state, worker.AdminWorkerHeavenState):
@@ -496,6 +516,13 @@ class Window(log.Loggable, gobject.GObject):
                     return
                 self.debug('default flow started')
                 value.addListener(self)
+        elif isinstance(state, planet.AdminComponentState):
+            name = state.get('name')
+            self.debug('stateAppend on component state of %s' % name)
+            if key == 'messages':
+                current = self.components_view.get_selected_name()
+                if name == current:
+                    self._messages_view.add_message(value)
         else:
             self.warning('stateAppend on unknown object %r' % state)
 
@@ -524,6 +551,13 @@ class Window(log.Loggable, gobject.GObject):
         elif isinstance(state, planet.AdminPlanetState):
             self.debug('something got removed from the planet')
             pass
+        elif isinstance(state, planet.AdminComponentState):
+            name = state.get('name')
+            self.debug('stateRemove on component state of %s' % name)
+            if key == 'messages':
+                current = self.components_view.get_selected_name()
+                if name == current:
+                    self._messages_view.clear_message(value.id)
         else:
             self.warning('stateRemove of key %s and value %r on unknown object %r' % (key, value, state))
 
@@ -534,13 +568,15 @@ class Window(log.Loggable, gobject.GObject):
             self._disconnected_dialog.destroy()
             self._disconnected_dialog = None
 
-        self.window.set_title('%s@%s:%d - Flumotion Administration'
-                              % (admin.user, admin.host, admin.port))
+        # FIXME: have a method for this
+        self.window.set_title(_('%s - Flumotion Administration') %
+            self.admin.adminInfoStr())
+        self._trayicon.set_tooltip(self.admin.adminInfoStr())
 
         self.emit('connected')
 
         # get initial info we need
-        self.setPlanetState(self.admin.getPlanetState())
+        self.setPlanetState(self.admin.planet)
 
         if not self._components:
             self.debug('no components detected, running wizard')
@@ -549,7 +585,7 @@ class Window(log.Loggable, gobject.GObject):
             self.runWizard()
     
     def admin_disconnected_cb(self, admin):
-        message = "Lost connection to manager, reconnecting ..."
+        message = _("Lost connection to manager, reconnecting ...")
         d = gtk.MessageDialog(self.window, gtk.DIALOG_DESTROY_WITH_PARENT,
             gtk.MESSAGE_WARNING, gtk.BUTTONS_NONE, message)
         # FIXME: move this somewhere
@@ -568,29 +604,35 @@ class Window(log.Loggable, gobject.GObject):
         elif id == 1:
             self.admin.reconnect()
         
-    def admin_connection_refused_later(self, host, port, use_insecure):
-        message = "Connection to manager on %s:%d (%s) was refused." \
-                  % (host, port, use_insecure and 'http' or 'https')
+    def admin_connection_refused_later(self, admin):
+        message = _("Connection to manager on %s was refused.") % \
+            admin.connectionInfoStr()
+        self._trayicon.set_tooltip(_("Connection to %s was refused") %
+            self.admin.adminInfoStr())
         self.info(message)
         d = dialogs.ErrorDialog(message, self)
         d.show_all()
         d.connect('response', self.close)
 
-    def admin_connection_refused_cb(self, admin, host, port, use_insecure):
+    def admin_connection_refused_cb(self, admin):
         log.debug('adminclient', "handling connection-refused")
-        reactor.callLater(0, self.admin_connection_refused_later,
-                          host, port, use_insecure)
+        reactor.callLater(0, self.admin_connection_refused_later, admin)
         log.debug('adminclient', "handled connection-refused")
 
-    def admin_ui_state_changed_cb(self, admin, name, state):
-        # called when the admin UI for that component has changed
-        current = self.components_view.get_selected_name()
-        if current != name:
-            return
+    def admin_connection_failed_later(self, admin, reason):
+        message = (_("Connection to manager on %s failed (%s).")
+                   % (admin.connectionInfoStr(), reason))
+        self._trayicon.set_tooltip("Connection to %s failed" %
+            self.admin.adminInfoStr())
+        self.info(message)
+        d = dialogs.ErrorDialog(message, self.window)
+        d.show_all()
+        d.connect('response', self.close)
 
-        comp = self.current_component
-        if comp:
-            comp.setUIState(state)
+    def admin_connection_failed_cb(self, admin, reason):
+        log.debug('adminclient', "handling connection-failed")
+        reactor.callLater(0, self.admin_connection_failed_later, admin, reason)
+        log.debug('adminclient', "handled connection-failed")
 
     # FIXME: deprecated
     def property_changed_cb(self, admin, componentName, propertyName, value):
@@ -617,6 +659,7 @@ class Window(log.Loggable, gobject.GObject):
 
     def update_components(self):
         self.components_view.update(self._components)
+        self._trayicon.update(self._components)
 
     def _set_stop_start_component_sensitive(self):
         state = self.current_component_state
@@ -642,14 +685,37 @@ class Window(log.Loggable, gobject.GObject):
         self._set_stop_start_component_sensitive()
 
         if not state:
+            self.debug('no state, returning')
             return
 
         name = state.get('name')
+        mood = state.get('mood')
+        messages = state.get('messages')
+        self._messages_view.clear()
+        # no ui, clear; FIXME: do this nicer
+        old = self.hpaned.get_child2()
+        self.hpaned.remove(old)
+        #sub = gtk.Label('%s does not have a UI yet' % name)
+        sub = gtk.Label("")
+        self.hpaned.add2(sub)
+        sub.show()
+
+        if messages:
+            for m in messages:
+                self.debug('have message %r' % m)
+                self._messages_view.add_message(m)
+
+        if mood == moods.sad.value:
+            self.debug('component %s is sad' % name)
+            self.statusbar.set('main',
+                _("Component %s is sad") % name)
+           
+            return
 
         def gotEntryCallback(result):
             entryPath, filename, methodName = result
 
-            self.statusbar.set('main', 'Showing UI for %s' % name)
+            self.statusbar.set('main', _('Showing UI for %s') % name)
 
             filepath = os.path.join(entryPath, filename)
             self.debug("Got the UI, lives in %s" % filepath)
@@ -668,30 +734,23 @@ class Window(log.Loggable, gobject.GObject):
         def gotEntryNoBundleErrback(failure):
             failure.trap(errors.NoBundleError)
 
-            self.statusbar.set('main', "No UI for component %s" % name)
-
-            # no ui, clear; FIXME: do this nicer
-            old = self.hpaned.get_child2()
-            self.hpaned.remove(old)
-            #sub = gtk.Label('%s does not have a UI yet' % name)
-            sub = gtk.Label("")
-            self.hpaned.add2(sub)
-            sub.show()
+            self.debug("No UI for component %s" % name)
+            self.statusbar.set('main', _("No UI for component %s") % name)
 
         def gotEntrySleepingComponentErrback(failure):
             failure.trap(errors.SleepingComponentError)
 
-            self.statusbar.set('main', "Component %s is still sleeping" % name)
+            self.statusbar.set('main',
+                _("Component %s is still sleeping") % name)
 
-            # no ui, clear; FIXME: do this nicer
-            old = self.hpaned.get_child2()
-            self.hpaned.remove(old)
-            #sub = gtk.Label('%s does not have a UI yet' % name)
-            sub = gtk.Label("")
-            self.hpaned.add2(sub)
-            sub.show()
-                      
-        self.statusbar.set('main', "Requesting UI for %s ..." % name)
+        self.statusbar.set('main', _("Requesting UI for %s ...") % name)
+        # if there's a current component being shown, give it a chance
+        # to clean up
+        if self.current_component:
+            if hasattr(self.current_component, 'cleanup'):
+                self.debug('Cleaning up current component view')
+                self.current_component.cleanup()
+        self.current_component = None
 
         d = self.admin.getEntry(state, 'admin/gtk')
         d.addCallback(gotEntryCallback)
@@ -737,12 +796,11 @@ class Window(log.Loggable, gobject.GObject):
         state = self.admin.getWorkerHeavenState()
         if not state.get('names'):
             self.show_error_dialog(
-                'The wizard cannot be run because no workers are logged in.')
+                _('The wizard cannot be run because no workers are logged in.'))
             return
         
         wiz = wizard.Wizard(self.window, self.admin)
         wiz.connect('finished', _wizard_finished_cb)
-        wiz.load_steps()
         wiz.run(True, state, False)
 
         self.wizard = wiz
@@ -779,7 +837,7 @@ class Window(log.Loggable, gobject.GObject):
             return
 
         dialog = dialogs.ProgressDialog("Reloading",
-            "Reloading component code for %s" % name, self.window)
+            _("Reloading component code for %s") % name, self.window)
         d = self.admin.reloadComponent(name)
         d.addCallback(lambda result, d: d.destroy(), dialog)
         # add error
@@ -787,16 +845,30 @@ class Window(log.Loggable, gobject.GObject):
         dialog.start()
 
     def _component_stop(self, state):
-        return self._component_do(state, 'stop', 'Stopping', 'Stopped')
+        """
+        @returns: a L{twisted.internet.defer.Deferred}
+        """
+        return self._component_do(state, 'Stop', 'Stopping', 'Stopped')
         
     def _component_start(self, state):
-        return self._component_do(state, 'start', 'Starting', 'Started')
+        """
+        @returns: a L{twisted.internet.defer.Deferred}
+        """
+        return self._component_do(state, 'Start', 'Starting', 'Started')
  
+    def _component_restart(self, state):
+        """
+        @returns: a L{twisted.internet.defer.Deferred}
+        """
+        d = self._component_stop(state)
+        d.addCallback(lambda r: self._component_start(state))
+        return d
+        
     def _component_do(self, state, action, doing, done):
         if not state:
             state = self.components_view.get_selected_state()
             if not state:
-                self.statusbar.push('main', "No component selected.")
+                self.statusbar.push('main', _("No component selected."))
                 return None
 
         name = state.get('name')
@@ -804,7 +876,7 @@ class Window(log.Loggable, gobject.GObject):
             return None
 
         mid = self.statusbar.push('main', "%s component %s" % (doing, name))
-        d = self.admin.componentCallRemote(state, action)
+        d = self.admin.callRemote('component'+action, state)
 
         def _actionCallback(result, self, mid):
             self.statusbar.remove('main', mid)
@@ -813,7 +885,7 @@ class Window(log.Loggable, gobject.GObject):
             self.statusbar.remove('main', mid)
             self.warning("Failed to %s component %s: %s" % (
                 action, name, failure))
-            self.statusbar.push('main', "Failed to %s component %s" % (
+            self.statusbar.push('main', _("Failed to %s component %s") % (
                 action, name))
             
         d.addCallback(_actionCallback, self, mid)
@@ -828,7 +900,7 @@ class Window(log.Loggable, gobject.GObject):
         self.on_open_connection(state)
 
     def file_open_cb(self, button):
-        d = connections.ConnectionsDialog(self.window)
+        d = gtkconnections.ConnectionsDialog(self.window)
         d.show()
         d.connect('have-connection', self.on_have_connection)
     
@@ -840,7 +912,7 @@ class Window(log.Loggable, gobject.GObject):
         d.destroy()
 
     def file_import_configuration_cb(self, button):
-        d = gtk.FileChooserDialog("Import Configuration...", self.window,
+        d = gtk.FileChooserDialog(_("Import Configuration..."), self.window,
                                   gtk.FILE_CHOOSER_ACTION_OPEN,
                                   (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                                    gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
@@ -848,20 +920,34 @@ class Window(log.Loggable, gobject.GObject):
         d.show()
         d.connect('response', self.on_import_response)
     
-    def getConfiguration_cb(self, conf_xml, name):
-        f = open(name, 'w')
-        f.write(conf_xml)
-        f.close()
+    def getConfiguration_cb(self, conf_xml, name, chooser):
+        file_exists = True
+        if os.path.exists(name):
+            d = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,
+                                  gtk.MESSAGE_ERROR, gtk.BUTTONS_YES_NO,
+                                  _("File already exists.\nOverwrite?"))
+            d.connect("response", lambda self, response: d.hide())
+            if d.run() == gtk.RESPONSE_YES:
+                file_exists = False
+        else:
+            file_exists = False
+
+        if not file_exists:
+            f = open(name, 'w')
+            f.write(conf_xml)
+            f.close()
+            chooser.destroy()
 
     def on_export_response(self, d, response):
         if response==gtk.RESPONSE_ACCEPT:
             deferred = self.admin.getConfiguration()
             name = d.get_filename()
-            deferred.addCallback(self.getConfiguration_cb, name)
-        d.destroy()
+            deferred.addCallback(self.getConfiguration_cb, name, d)
+        else:
+            d.destroy()
 
     def file_export_configuration_cb(self, button):
-        d = gtk.FileChooserDialog("Export Configuration...", self.window,
+        d = gtk.FileChooserDialog(_("Export Configuration..."), self.window,
                                   gtk.FILE_CHOOSER_ACTION_SAVE,
                                   (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                                    gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
@@ -905,7 +991,8 @@ class Window(log.Loggable, gobject.GObject):
             failure.trap(errors.ReloadSyntaxError)
             _stop(progress)
             self.show_error_dialog(
-                "Could not reload component:\n%s." % failure.getErrorMessage())
+                _("Could not reload component:\n%s.") %
+                failure.getErrorMessage())
             return None
             
         def _callLater(admin, dialog):
@@ -914,10 +1001,10 @@ class Window(log.Loggable, gobject.GObject):
             deferred.addErrback(_syntaxErrback, self, dialog)
             deferred.addErrback(self._defaultErrback)
         
-        dialog = dialogs.ProgressDialog("Reloading ...",
-            "Reloading client code", self.window)
+        dialog = dialogs.ProgressDialog(_("Reloading ..."),
+            _("Reloading client code"), self.window)
         l = lambda admin, text, dialog: dialog.message(
-            "Reloading %s code" % text)
+            _("Reloading %s code") % text)
         self.admin.connect('reloading', l, dialog)
         dialog.start()
         reactor.callLater(0.2, _callLater, self.admin, dialog)
@@ -930,7 +1017,7 @@ class Window(log.Loggable, gobject.GObject):
         code.interact()
 
     def help_about_cb(self, button):
-        dialog = gtk.Dialog('About Flumotion', self.window,
+        dialog = gtk.Dialog(_('About Flumotion'), self.window,
                             gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                             (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
         dialog.set_has_separator(False)
@@ -949,12 +1036,12 @@ class Window(log.Loggable, gobject.GObject):
         version.set_use_markup(True)
         version.show()
 
-        text = 'Flumotion is a streaming media server\n\n(C) 2004-2005 Fluendo S.L.'
+        text = _('Flumotion is a streaming media server.\n\n(C) 2004-2005 Fluendo S.L.')
         authors = ('Andy Wingo',
                    'Johan Dahlin',
                    'Thomas Vander Stichele',
                    'Wim Taymans')
-        text += '\n\n<small>Authors:\n'
+        text += '\n\n<small>' + _('Authors') + ':\n'
         for author in authors:
             text += '  %s\n' % author
         text += '</small>'
@@ -972,8 +1059,7 @@ class Window(log.Loggable, gobject.GObject):
 
     def show(self):
         # XXX: Use show()
-        self.window.show_all()
-        
+        self.window.show()
 
-gobject.type_register(Window)
+pygobject.type_register(Window)
 
