@@ -2,7 +2,8 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.
+# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
 # the GNU General Public License version 2 as published by
@@ -23,7 +24,6 @@ small common functions used by all processes
 """
 
 import errno
-import glob
 import os 
 import socket
 import sys
@@ -31,10 +31,9 @@ import time
 import signal
 
 from twisted.python import reflect, rebuild, components
-from twisted.spread import pb
 from twisted.internet import address
 
-from flumotion.common import errors, log
+from flumotion.common import log
 
 # Note: This module is loaded very early on, so
 #       don't add any extra flumotion imports unless you
@@ -128,7 +127,7 @@ def version(binary):
     block = []
     block.append("%s %s" % (binary, configure.version))
     block.append("part of Flumotion - a streaming media server")
-    block.append("(C) Copyright 2004,2005 Fluendo")
+    block.append("(C) Copyright 2004,2005,2006 Fluendo")
     return "\n".join(block)
              
 def mergeImplements(*classes):
@@ -215,261 +214,6 @@ def argRepr(args=(), kwargs={}, max=-1):
             
     return s
 
-def _listDirRecursively(path):
-    """
-    I'm similar to os.listdir, but I work recursively and only return
-    directories containing python code.
-    
-    @param path: the path
-    @type  path: string
-    """
-    retval = []
-    # files are never returned, only directories
-    if not os.path.isdir(path):
-        return retval
-
-    try:
-        files = os.listdir(path)
-    except OSError:
-        pass
-    else:
-        for f in files:
-            # this only adds directories since files are not returned
-            retval += _listDirRecursively(os.path.join(path, f))
-
-    if glob.glob(os.path.join(path, '*.py*')):
-        retval.append(path)
-            
-    return retval
-
-def _listPyFileRecursively(path):
-    """
-    I'm similar to os.listdir, but I work recursively and only return
-    files representing python non-package modules.
-    
-    @param path: the path
-    @type  path: string
-
-    @rtype:      list
-    @returns:    list of files underneath the given path containing python code
-    """
-    retval = []
-
-    # get all the dirs containing python code
-    dirs = _listDirRecursively(path)
-
-    for dir in dirs:
-        pyfiles = glob.glob(os.path.join(dir, '*.py*'))
-        dontkeep = glob.glob(os.path.join(dir, '*__init__.py*'))
-        for f in dontkeep:
-            if f in pyfiles:
-                pyfiles.remove(f)
-
-        retval.extend(pyfiles)
-
-    return retval
-
-def _findPackageCandidates(path, prefix='flumotion.'):
-    """
-    I take a directory and return a list of candidate python packages.
-    A package is a module containing modules; typically the directory
-    with the same name as the package contains __init__.py
-
-    @param path: the path
-    @type  path: string
-    """
-    # this function also "guesses" candidate packages when __init__ is missing
-    # so a bundle with only a subpackage is also detected
-    dirs = _listDirRecursively(path)
-    if path in dirs:
-        dirs.remove(path)
-
-    # chop off the base path to get a list of "relative" bundlespace paths
-    bundlePaths = [x[len(path) + 1:] for x in dirs]
-
-    # remove some common candidates, like .svn subdirs, or containing -
-    isNotSvn = lambda x: x.find('.svn') == -1
-    bundlePaths = filter(isNotSvn, bundlePaths)
-    isNotDashed = lambda x: x.find('-') == -1
-    bundlePaths = filter(isNotDashed, bundlePaths)
-
-    # convert paths to module namespace
-    bundlePackages = [".".join(x.split(os.path.sep)) for x in bundlePaths]
-
-    # remove all not starting with prefix
-    isInPrefix = lambda x: x.startswith(prefix)
-    bundlePackages = filter(isInPrefix, bundlePackages)
-
-    # sort them so that depending packages are after higher-up packages
-    bundlePackages.sort()
-        
-    return bundlePackages
-
-def _findEndModuleCandidates(path, prefix='flumotion.'):
-    """
-    I take a directory and return a list of candidate end python modules.
-    These are non-package modules.
-
-    @param path: the path
-    @type  path: string
-    """
-    files = _listPyFileRecursively(path)
-
-    # chop off the base path to get a list of "relative" bundlespace paths
-    bundlePaths = [x[len(path) + 1:] for x in files]
-
-    # remove some common candidates, like .svn subdirs, or containing -
-    isNotSvn = lambda x: x.find('.svn') == -1
-    bundlePaths = filter(isNotSvn, bundlePaths)
-    isNotDashed = lambda x: x.find('-') == -1
-    bundlePaths = filter(isNotDashed, bundlePaths)
-
-    # convert paths to module namespace
-    bundleModules = [pathToModuleName(x) for x in bundlePaths]
-
-    # remove all not starting with prefix
-    isInPrefix = lambda x: x.startswith(prefix)
-    bundleModules = filter(isInPrefix, bundleModules)
-
-    # sort them so that depending packages are after higher-up packages
-    bundleModules.sort()
-
-    # make unique
-    res = {}
-    for b in bundleModules: res[b] = 1
-
-    return res.keys()
-
-# FIXME: an extra key argument (used for bundle) might help in keeping
-# track of old paths
-# ie, it would ensure that only one packagePath per key is registered
-def registerPackagePath(packagePath, prefix='flumotion'):
-    """
-    Register a given path as a path that can be imported from.
-    Used to support partition of bundled code or import code from various
-    uninstalled location.
-
-    sys.path will also be changed to include this, and remove references
-    to older packagePath's for the same bundle.
-
-    @param packagePath: path to add under which the module namespaces live,
-                        (ending in an md5sum, for flumotion purposes)
-    @type  packagePath: string
-    @param prefix:      prefix of the packages to be considered
-    @type  prefix:      string
-    """
-
-    # FIXME: this should potentially also clean up older registered package
-    # paths for the same bundle ?
-    # This would involve us keeping track of what has been registered before,
-    # and would probably involve creating an object to keep track of this state
-
-    # First add the root to sys.path, so we can import stuff from it,
-    # probably a good idea to live it there, if we want to do
-    # fancy stuff later on.
-    packagePath = os.path.abspath(packagePath)
-    if not os.path.exists(packagePath):
-        log.warning('bundle', 'registering a non-existing package path %s' %
-            packagePath)
-
-    log.log('bundle', 'registering packagePath %s' % packagePath)
-
-    # check if a packagePath for this bundle was already registered
-    # by stripping off the last part, which is the md5sum
-    oneup = os.path.split(packagePath)[0]
-    if oneup:
-        paths = sys.path
-        targets = [x for x in paths if x.startswith(oneup) and x != packagePath]
-
-    for path in targets:
-        log.log('bundle', 'removing old packagePath %s from sys.path' % path)
-        sys.path.remove(path)
-
-    # put packagePath at the top of sys.path if not in there
-    if not packagePath in sys.path:
-        log.log('bundle', 'adding packagePath %s to sys.path' % packagePath)
-        sys.path.insert(0, packagePath)
-
-    # Find the packages in the path and sort them,
-    # the following algorithm only works if they're sorted.
-    # By sorting the list we can ensure that a parent package
-    # is always processed before one of its children
-    packageNames = _findPackageCandidates(packagePath, prefix)
-    packageNames.sort()
-
-    if not packageNames:
-        log.log('bundle', 'packagePath %s does not have package candidates' %
-            packagePath)
-        return
-
-    log.log('bundle', 'package candidates %r' % packageNames)
-    # Since the list is sorted, the top module is the first item
-    log.log('bundle', 'packagePath %s has packageNames %r' % (
-        packagePath, packageNames)) 
-
-    toplevelName = packageNames[0]
-    
-    # Insert or move the bundle's absolute path to the top of __path__ of
-    # each of its higher-level packages, so reload() will take the new path
-
-    # FIXME: for complete correctness, it'd be good to remove the path
-    # for the previous bundle if there is a previous bundle
-    partials = []
-    for partial in toplevelName.split("."):
-        partials.append(partial)
-        name = ".".join(partials)
-        try:
-            package = reflect.namedAny(name)
-        #except ValueError: # Empty module name, ie. subdir has no __init__
-        #    continue
-        except:
-            print "ERROR: could not reflect name %s" % name
-            raise
-        path = os.path.join(packagePath, name.replace('.', os.sep))
-        if path in package.__path__:
-            package.__path__.remove(path)
-        package.__path__.insert(0, path)
-        
-    for packageName in packageNames[1:]:
-        package = sys.modules.get(packageName, None)
-        
-        # If the package fails to import from our bundle, it means
-        # that it's unknown at the moment, import it from the package dir
-        # (eg non bundle)
-        if not package:
-            package = reflect.namedAny(packageName)
-
-        # Append ourselves to the packages __path__, this is all
-        # magic that's required
-        subPath = os.path.join(packagePath,
-                               packageName.replace('.', os.sep))
-
-        # rebuild the package
-        rebuild.rebuild(package)
-
-        # insert at front because FLU_REGISTRY_PATH paths should override
-        # base components, and because subsequent reload() should prefer
-        # the latest registered path
-        # FIXME: we might eventually want to remove old paths for the same
-        # bundle   
-        if subPath in package.__path__:
-            log.log('bundle', 'moving subPath %s to top for package %r' % (
-                subPath, package))
-            package.__path__.remove(subPath)
-            package.__path__.insert(0, subPath)
-        else:
-            log.log('bundle', 'inserting subPath %s for package %r' % (
-                subPath, package))
-            package.__path__.insert(0, subPath)
-
-    # now rebuild all non-package modules in this packagePath
-    moduleNames = _findEndModuleCandidates(packagePath)
-    for name in moduleNames:
-        if name in sys.modules:
-            log.log('bundle', "rebuilding non-package module %s" % name)
-            module = reflect.namedAny(name)
-            rebuild.rebuild(module)
-
 def ensureDir(dir, description):
     """
     Ensure the given directory exists, creating it if not.
@@ -479,20 +223,24 @@ def ensureDir(dir, description):
         try:
             os.makedirs(dir)
         except:
+            from flumotion.common import errors
             raise errors.SystemError, "could not create %s directory %s" % (
                 description, dir)
 
+def getPidPath(type, name):
+    """
+    Get the full path to the pid file for the given process type and name.
+    """
+    return os.path.join(configure.rundir, '%s.%s.pid' % (type, name))
  
 def writePidFile(type, name):
     """
     Write a pid file in the run directory, using the given process type
     and process name for the filename.
     """
-    
     ensureDir(configure.rundir, "rundir")
     pid = os.getpid()
-    pidPath = os.path.join(configure.rundir, '%s.%s.pid' % (type, name))
-    file = open(pidPath, 'w')
+    file = open(getPidPath(type, name), 'w')
     file.write("%d\n" % pid)
     file.close()
  
@@ -501,19 +249,17 @@ def deletePidFile(type, name):
     Delete the pid file in the run directory, using the given process type
     and process name for the filename.
     """
-    
-    pidPath = os.path.join(configure.rundir, '%s.%s.pid' % (type, name))
-    os.unlink(pidPath)
+    os.unlink(getPidPath(type, name))
  
 def getPid(type, name):
     """
     Get the pid from the pid file in the run directory, using the given
     process type and process name for the filename.
     
-    @returns: pid of the process, or None.
+    @returns: pid of the process, or None if not running or file not found.
     """
     
-    pidPath = os.path.join(configure.rundir, '%s.%s.pid' % (type, name))
+    pidPath = getPidPath(type, name)
     if not os.path.exists(pidPath):
         return
     
@@ -571,24 +317,20 @@ def checkPidRunning(pid):
  
 def waitPidFile(type, name):
     """
-    Wait for the given process type and name to have started.
-    Return the pid if it started successfully, or None if it didn't.
+    Wait for the given process type and name to have started and created
+    a pid file.
+
+    Return the pid.
     """
+    # getting it from the start avoids an unneeded time.sleep
+    pid = getPid(type, name)
     
-    mtime = os.stat(configure.rundir)[8]
-    pid = getPid(type, name)
-    if pid:
-        return pid
-         
-    while os.stat(configure.rundir)[8] == mtime:
-        pid = getPid(type, name)
-        #if pid:
-        #    return pid
+    while not pid:
         time.sleep(0.1)
- 
-    pid = getPid(type, name)
+        pid = getPid(type, name)
+
     return pid
- 
+
 def waitForTerm():
     """
     Wait until we get killed by a TERM signal (from someone else).
@@ -663,11 +405,16 @@ def checkRemotePort(host, port):
 
 def addressGetHost(a):
     """
-    Get the port number of an IPv4 address.
+    Get the host name of an IPv4 address.
 
     @type a: L{twisted.internet.address.IPv4Address}
     """
-    assert(isinstance(a, address.IPv4Address))
+    if not isinstance(a, address.IPv4Address) and not isinstance(a,
+        address.UNIXAddress):
+        raise TypeError("object %r is not an IPv4Address or UNIXAddress" % a)
+    if isinstance(a, address.UNIXAddress):
+        return 'localhost'
+
     try:
         host = a.host
     except AttributeError:
@@ -705,11 +452,153 @@ def pathToModuleName(path):
     """
     Convert the given (relative) path to the python module it would have to
     be imported as.
+
+    Return None if the path is not a valid python module
     """
-    if path.endswith('.pyc'): path = path[:-4]
-    if path.endswith('.py'): path = path[:-3]
-    if path.endswith('__init__'): path = path[:-9]
+    # __init__ is last because it works on top of the first three
+    valid = False
+    suffixes = ['.pyc', '.pyo', '.py', os.path.sep + '__init__']
+    for s in suffixes:
+        if path.endswith(s):
+            path = path[:-len(s)]
+            valid = True
+
+    # if the path still contains dots, it can't possibly be a valid module
+    if not '.' in path:
+        valid = True
+
+    if not valid:
+        return None
 
     return ".".join(path.split(os.path.sep))
 
+def getLL():
+    """
+    Return the (at most) two-letter language code set for message translation.
+    """
+    # LANGUAGE is a GNU extension; it can be colon-seperated but we ignore the
+    # advanced stuff. If that's not present, just use LANG, as normal.
+    language = os.environ.get('LANGUAGE', None)
+    if language != None:
+      LL = language[:2]
+    else:
+      lang = os.environ.get('LANG', 'en')
+      LL = lang[:2]
 
+    return LL
+
+def gettexter(domain):
+    """
+    Returns a method you can use as _ to translate strings for the given
+    domain.
+    """
+    import gettext
+    return lambda s: gettext.dgettext(domain, s)
+
+def compareVersions(first, second):
+    """
+    Compares two version strings.  Returns -1, 0 or 1 if first is smaller than,
+    equal to or larger than second.
+
+    @type  first:  str
+    @type  second: str
+
+    @rtype: int
+    """
+    if first == second:
+        return 0
+
+    firsts = first.split(".")
+    seconds = second.split(".")
+
+    while firsts or seconds:
+        f = 0
+        s = 0
+        try:
+            f = int(firsts[0])
+            del firsts[0]
+        except IndexError:
+            pass
+        try:
+            s = int(seconds[0])
+            del seconds[0]
+        except IndexError:
+            pass
+
+        if f < s:
+            return -1
+        if f > s:
+            return 1
+
+    return 0
+
+def _uniq(l, key=lambda x: x):
+    """
+    Filters out duplicate entries in a list.
+    """
+    out = []
+    for x in l:
+        if key(x) not in [key(y) for y in out]:
+            out.append(x)
+    return out
+
+def _call_each_method(obj, method, mro, args, kwargs):
+    procs = []
+    for c in mro:
+        if hasattr(c, method):
+            proc = getattr(c, method)
+            assert callable(proc) and hasattr(proc, 'im_func'),\
+                   'attr %s of class %s is not a method' % (method, c)
+            procs.append(proc)
+
+    # In a hierarchy A -> B, if A implements the method, B will inherit
+    # it as well. Compare the functions implementing the methods so as
+    # to avoid calling them twice.
+    procs = _uniq(procs, lambda proc: proc.im_func)
+
+    for proc in procs:
+        proc(obj, *args, **kwargs)
+
+def call_each_method(obj, method, *args, **kwargs):
+    """
+    Invoke all implementations of a method on an object.
+
+    Searches for method implementations in the object's class and all of
+    the class' superclasses. Calls the methods in method resolution
+    order, which goes from subclasses to superclasses.
+    """
+    mro = type(obj).__mro__
+    _call_each_method(obj, method, mro, args, kwargs)
+
+def call_each_method_reversed(obj, method, *args, **kwargs):
+    """
+    Invoke all implementations of a method on an object.
+
+    Like call_each_method, but calls the methods in reverse method
+    resolution order, from superclasses to subclasses.
+    """
+    # do a list() so as to copy the mro, we reverse the list in
+    # place so as to start with the base class
+    mro = list(type(obj).__mro__)
+    mro.reverse()
+    _call_each_method(obj, method, mro, args, kwargs)
+    
+class InitMixin(object):
+    """
+    A mixin class to help with object initialization.
+
+    In some class hierarchies, __init__ is only used for initializing
+    instance variables. In these cases it is advantageous to avoid the
+    need to "chain up" to a parent implementation of a method. Adding
+    this class to your hierarchy will, for each class in the object's
+    class hierarchy, call the class's init() implementation on the
+    object.
+
+    Note that the function is called init() without underscrores, and
+    that there is no need to chain up to superclasses' implementations.
+
+    Uses call_each_method_reversed() internally.
+    """
+
+    def __init__(self, *args, **kwargs):
+        call_each_method_reversed(self, 'init', *args, **kwargs)
