@@ -2,7 +2,8 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.
+# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
 # the GNU General Public License version 2 as published by
@@ -28,17 +29,21 @@ __all__ = ['ComponentAvatar', 'ComponentHeaven']
 
 import time
 
-import gst
 from twisted.spread import pb
-from twisted.internet import reactor
+from twisted.internet import reactor, defer
 
 from flumotion.configure import configure
 # rename to base
 from flumotion.manager import base
 from flumotion.common import errors, interfaces, keycards, log, config, planet
+from flumotion.common import messages
 from flumotion.twisted import flavors
+from flumotion.twisted.defer import defer_generator_method
 from flumotion.common.planet import moods
 
+from flumotion.common.messages import N_
+T_ = messages.gettexter('flumotion')
+
 # abstracts the concept of a GStreamer tcpserversink (feeder) producing a feed
 class Feeder:
     """
@@ -130,6 +135,7 @@ class Feeder:
         return self.feederName
 
     def getListenHost(self):
+        # return what we think is the IP address where the component is running
         assert self.component
         return self.component.getClientAddress()
 
@@ -265,8 +271,6 @@ class ComponentAvatar(base.ManagerAvatar):
         self.starting = False
         self.jobState = None # retrieved after mind attached
         self.componentState = None # set by the vishnu by componentAttached
-        self._gstState = gst.STATE_NULL # FIXME: deprecate ?
-        self._gstOldState = gst.STATE_NULL # FIXME: deprecate ?
 
         self.logName = avatarId
 
@@ -280,22 +284,31 @@ class ComponentAvatar(base.ManagerAvatar):
     ### python methods
     def __repr__(self):
         if self.componentState:
-            mood = self.componentState.get('mood')
+            mood = moods.get(self.componentState.get('mood')).name
         else:
             mood = '(unknown)'
-        return '<%s %s in mood %s>' % (self.__class__.__name__,
-                                        self.avatarId, mood)
+        return '<%s %s (mood %s)>' % (self.__class__.__name__,
+                                      self.avatarId, mood)
 
     ### ComponentAvatar methods
     def cleanup(self):
         """
-        Clean up when detaching."
+        Clean up when detaching.
         """
         if self._HeartbeatCheckDC:
             self._HeartbeatCheckDC.cancel()
         self._HeartbeatCheckDC = None
 
+        if self.ports:
+            self.vishnu.releasePortsOnWorker(self.getWorkerName(),
+                                             self.ports.values())
+            
+        self.ports = {}
+
         self.jobState = None
+        # if we're sad, we need an explicit cleanup to acknowledge the problem
+        if not self._getMoodValue() == moods.sad.value:
+            self._setMood(moods.sleeping)
 
     def _heartbeatCheck(self):
         """
@@ -308,18 +321,10 @@ class ComponentAvatar(base.ManagerAvatar):
                 > self._heartbeatCheckInterval \
             and self._getMoodValue() != moods.lost.value:
                 self.warning('heartbeat missing, component is lost')
-                self._setMessage('Component %s is lost.' % self.avatarId)
                 self._setMood(moods.lost)
         self._HeartbeatCheckDC = reactor.callLater(self._heartbeatCheckInterval,
             self._heartbeatCheck)
 
-    # FIXME: this doesn't actually show up
-    def _setMessage(self, message):
-        if not self.componentState:
-            return
-
-        self.componentState.set('message', message)
-
     def _setMood(self, mood):
         if not self.componentState:
             return
@@ -337,6 +342,12 @@ class ComponentAvatar(base.ManagerAvatar):
             return
         return self.componentState.get('mood')
 
+    def _addMessage(self, message):
+        if not self.componentState:
+            return
+
+        self.componentState.append('messages', message)
+
     # general fallback for unhandled errors so we detect them
     # FIXME: we can't use this since we want a PropertyError to fall through
     # after going through the PropertyErrback.
@@ -372,11 +383,14 @@ class ComponentAvatar(base.ManagerAvatar):
         d.addErrback(self._mindPipelineErrback)
         #d.addErrback(self._mindErrback)
 
+        # FIXME: return d to serialize ?
+
     def _mindGetStateCallback(self, state): 
         # called after the mind has attached.
-        # state: L{flumotion.common.component.ManagerComponentState}
+        # state: L{flumotion.common.planet.ManagerJobState}
         if not state:
-            self.debug('no state received yet, rescheduling')
+            # how in god's name is this possible?
+            self.warning('no state received yet, rescheduling')
             reactor.callLater(1, self._getState)
             return None
             
@@ -406,7 +420,7 @@ class ComponentAvatar(base.ManagerAvatar):
  
     # IStateListener methods
     def stateSet(self, state, key, value):
-        self.debug("state set on %r: %s now %r" % (state, key, value))
+        self.log("state set on %r: %s now %r" % (state, key, value))
         if key == 'mood':
             self.info('Mood changed to %s' % moods.get(value).name)
 
@@ -444,6 +458,7 @@ class ComponentAvatar(base.ManagerAvatar):
         """
         # non-feed components don't have these keys
         if not self.jobState.hasKey('feederNames'):
+            self.warning('no feederNames key, so no feeders')
             return []
 
         return self.jobState.get('feederNames', [])
@@ -452,16 +467,19 @@ class ComponentAvatar(base.ManagerAvatar):
         """
         Returns the port this feed is being fed on.
         """
-        return self.ports[feedName]
+        return self.ports[self.getName() + ':' + feedName]
  
     def getRemoteManagerIP(self):
+        """
+        Get the IP address of the manager as seen by the component.
+        """
         return self.jobState.get('ip')
 
     def getWorkerName(self):
         """
         Return the name of the worker.
         """
-        return self.componentState.get('workerName')
+        return self.jobState.get('workerName')
 
     def getPid(self):
         return self.jobState.get('pid')
@@ -484,6 +502,20 @@ class ComponentAvatar(base.ManagerAvatar):
         d.addErrback(lambda x: None)
         return d
             
+    def setup(self, config):
+        """
+        Set up the component with the given config.
+        """
+        def _setupErrback(failure, self):
+            self._setMood(moods.sad)
+            return failure
+
+        self.debug('remote call setup(config=%r)' % config)
+        d = self.mindCallRemote('setup', config)
+        d.addErrback(_setupErrback, self)
+        return d
+
+        
     # This function tells the component to start
     # feedcomponents will start consuming feeds and start its feeders
     def start(self, eatersData, feedersData):
@@ -494,24 +526,60 @@ class ComponentAvatar(base.ManagerAvatar):
                            of elements feeding our eaters
         @type feedersData: tuple of (name, host) tuples of our feeding elements
         """
-        def startCallback(feedData):
-            if feedData:
-                for feedName, host, port in feedData:
-                    self.debug('feed %s (%s:%d) is ready' % (
-                        feedName, host, port))
-                    self.host = host
-                    self.ports[feedName] = port
-                    
-                    self.checkFeedReady(feedName)
+        self.debug('ComponentAvatar.start(eatersData=%r, feedersData=%r)' % (
+            eatersData, feedersData))
+        for feedname, host, port in feedersData:
+            if feedname in self.ports:
+                self.warning('feed %s already has port %d; trying to '
+                             'start anyway' % (feedname, self.ports[feedname]))
+            self.ports[feedname] = port
 
-            self.debug('startCallback: done starting')
+        config = self.componentState.get('config')
+        master = config['clock-master'] # avatarId of the clock master comp
+        clocking = None
+        if master:
+            self.debug('Need to synchronize with clock master %r' % master)
+            d = self.heaven.getMasterClockInfo(master, self.avatarId)
+            yield d
+            try:
+                clocking = d.value()
+                self.debug('Got master clock info %r' % (clocking, ))
+                host, port, base_time = clocking
+                # FIXME: the host we get is as seen from the component, so lo
+                # mangle it here
+                # if the clock master is local (which is what we assume for now)
+                # and the slave is not, then we need to tell the slave our
+                # IP
+                if (not self.heaven._componentIsLocal(self)
+                    and host == '127.0.0.1'):
+                    host = self.getRemoteManagerIP()
+                    self.debug('Overriding clock master host to %s' % host)
+                    clocking = (host, port, base_time)
 
-        def startErrback(reason):
-            self.error("Could not make component start, reason %s" % reason)
-                
-        d = self.mindCallRemote('start', eatersData, feedersData)
-        d.addCallback(startCallback)
-        d.addErrback(startErrback)
+                if master == self.avatarId:
+                    self.debug('we are the master, so reset to None')
+                    # we needed to wait for the set_master to complete,
+                    # but we don't slave the clock to itself...
+                    clocking = None
+            except Exception, e:
+                self.error("Could not make component start, reason %s"
+                           % log.getExceptionMessage(e))
+
+        self.debug('calling remote_start on component %r' % self)
+        d = self.mindCallRemote('start', eatersData, feedersData, clocking)
+        yield d
+        try:
+            d.value()
+        except Exception, e:
+            m = messages.Error(T_(N_("Could not start component.")),
+                debug = log.getExceptionMessage(e),
+                id = "component-start")
+            self._addMessage(m)
+            self.warning("Could not make component start, reason %s"
+                       % log.getExceptionMessage(e))
+            self._setMood(moods.sad)
+            raise e
+    start = defer_generator_method(start)
     
     def setElementProperty(self, element, property, value):
         """
@@ -591,48 +659,19 @@ class ComponentAvatar(base.ManagerAvatar):
         d.addErrback(self._mindErrback, errors.ReloadSyntaxError)
         return d
 
-    # FIXME rename to something that reflects an action, like startFeedIfReady
-    def checkFeedReady(self, feedName):
-        # check if the given feed is ready to start, and start it if it is
-        self.debug('checkFeedReady: feedName %s' % feedName)
-        print "THOMAS: self.ports.keys(): %r" % self.ports.keys()
-        if not self.ports.has_key(feedName):
-            self.debug('checkFeedReady: no port yet')
-            return
-        
-        if not self.getFeeders():
-            self.debug('checkFeedReady: no remote options yet')
-            return
-
-        if self._gstState == gst.STATE_PLAYING:
-            self.debug('checkFeedReady: setting to ready')
-            self.heaven.setFeederReadiness(self, feedName, True)
-            self.debug('checkFeedReady: set to ready')
-
-        elif self._gstState == gst.STATE_PAUSED and \
-            self._gstOldState == gst.STATE_PLAYING:
-            self.debug('checkFeedReady: setting to not ready')
-            self.heaven.setFeederReadiness(self, feedName, False)
-            self.debug('checkFeedReady: set to not ready')
-        else:
-            self.debug('checkFeedReady: no play/pause trans (%s to %s)' % (
-                      gst.element_state_get_name(self._gstState),
-                      gst.element_state_get_name(self._gstOldState)))
-        return
-
     # FIXME: maybe make a BouncerComponentAvatar subclass ?
     def authenticate(self, keycard):
         d = self.mindCallRemote('authenticate', keycard)
         d.addErrback(self._mindErrback)
         return d
 
-    def removeKeycard(self, keycardId):
+    def removeKeycardId(self, keycardId):
         """
         Remove a keycard managed by this bouncer because the requester
         has gone.
         """
-        self.debug('remotecalling removeKeycard with id %s' % keycardId)
-        d = self.mindCallRemote('removeKeycard', keycardId)
+        self.debug('remotecalling removeKeycardId with id %s' % keycardId)
+        d = self.mindCallRemote('removeKeycardId', keycardId)
         d.addErrback(self._mindErrback)
         return d
 
@@ -647,29 +686,26 @@ class ComponentAvatar(base.ManagerAvatar):
         return d
 
     ### IPerspective methods, called by the worker's component
-    def perspective_log(self, *msg):
-        log.debug(self.avatarId, *msg)
-        
+    def perspective_feedReady(self, feedName, isReady):
+        """
+        Called by the component to tell the manager that a given feed is
+        ready or not. Will notify other components depending on this
+        feeder, starting them if all of their dependencies are ready.
+
+        @param feedName: name of the feeder, e.g. "default".
+        @type  feedName: string
+        @param isReady: True if the feed is now ready, False otherwise.
+        @type  isReady: bool
+        """
+        assert isinstance(feedName, str)
+        self.heaven.setFeederReadiness(self, feedName, isReady)
+            
     def perspective_heartbeat(self, moodValue):
         self.lastHeartbeat = time.time()
         #log.log(self.avatarId,
         #    "got heartbeat at %d" % int(self.lastHeartbeat))
         self._setMoodValue(moodValue)
 
-    def perspective_feedStateChanged(self, feedName, oldState, newState):
-        self.debug('feedStateChanged: feed name %s, old %s, new %s' % (
-            feedName, gst.element_state_get_name(oldState),
-            gst.element_state_get_name(newState)))
-        self._gstState = newState
-        self._gstOldState = oldState
-        
-        if newState == gst.STATE_PLAYING:
-            self.debug('%r is now playing' % self)
-            self.checkFeedReady(feedName)
-        if oldState == gst.STATE_PLAYING:
-            self.debug('%r was playing' % self)
-            self.checkFeedReady(feedName)
-            
     def perspective_error(self, element, error):
         self.error('error element=%s string=%s' % (element, error))
         self.heaven.removeComponent(self)
@@ -681,9 +717,6 @@ class ComponentAvatar(base.ManagerAvatar):
         self.vishnu.adminHeaven.avatarsCallRemote("componentCall",
             componentState, methodName, *args, **kwargs)
 
-    def perspective_notifyFeedPorts(self, feedPorts):
-        self.debug('received feed ports from component: %s' % feedPorts)
-
     def perspective_authenticate(self, bouncerName, keycard):
         """
         Authenticate the given keycard using the given bouncer.
@@ -702,7 +735,7 @@ class ComponentAvatar(base.ManagerAvatar):
         bouncerAvatar = self.heaven.getAvatar(avatarId)
         return bouncerAvatar.authenticate(keycard)
 
-    def perspective_removeKeycard(self, bouncerName, keycardId):
+    def perspective_removeKeycardId(self, bouncerName, keycardId):
         """
         Remove a keycard on the given bouncer on behalf of a component's medium.
 
@@ -719,7 +752,7 @@ class ComponentAvatar(base.ManagerAvatar):
             raise errors.UnknownComponentError(avatarId)
 
         bouncerAvatar = self.heaven.getAvatar(avatarId)
-        return bouncerAvatar.removeKeycard(keycardId)
+        return bouncerAvatar.removeKeycardId(keycardId)
 
     def perspective_expireKeycard(self, requesterName, keycardId):
         """
@@ -738,6 +771,16 @@ class ComponentAvatar(base.ManagerAvatar):
         componentAvatar = self.heaven.getAvatar(requesterName)
         return componentAvatar.expireKeycard(keycardId)
 
+    def perspective_reservePortsOnWorker(self, workerName, number_of_ports):
+        """
+        Request a number of ports on a particular worker.
+
+        This can be called from a job if it needs some ports itself
+        """
+        ports = self.heaven.vishnu.reservePortsOnWorker(workerName, 
+            number_of_ports)
+        return ports
+
 class ComponentHeaven(base.ManagerHeaven):
     """
     I handle all registered components and provide avatars for them.
@@ -755,13 +798,17 @@ class ComponentHeaven(base.ManagerHeaven):
         """
         base.ManagerHeaven.__init__(self, vishnu)
         self._feederSets = {}
-        # FIXME: deprecate componentEntries
-        self._componentEntries = {} # configuration entries
+
+        # hash of clock master avatarId ->
+        # list of (deferreds, avatarId) created by getMasterClockInfo
+        self._clockMasterWaiters = {}
+        self._masterClockInfo = {}
         
-   
     ### our methods
     def _componentIsLocal(self, componentAvatar):
+        # gets what we think is the other side's address
         host = componentAvatar.getClientAddress()
+
         if host == '127.0.0.1':
             return True
         else:
@@ -787,17 +834,6 @@ class ComponentHeaven(base.ManagerHeaven):
         """
         self.removeAvatar(componentAvatar.avatarId)
         
-    # FIXME: deprecate this completely
-    def loadConfiguration(self, filename, string=None):
-        conf = config.FlumotionConfigXML(filename, string)
-        
-        # get atmosphere and flow entries
-        entries = conf.getComponentEntries()
-        self.debug('got entries %r from conf %r' % (entries, conf))
-        self._componentEntries.update(entries)
-        self.debug("added entries for components %r" %
-            self._componentEntries.keys())
-            
     def _getComponentEatersData(self, componentAvatar):
         """
         Retrieve the information about the feeders this component's eaters
@@ -811,7 +847,7 @@ class ComponentHeaven(base.ManagerHeaven):
         """
 
         eaterFeederNames = componentAvatar.getEaters()
-        self.debug('eaterFeederNames: %r' % eaterFeederNames)
+        self.debug('feeders we eat from: %r' % eaterFeederNames)
 
         #feederName is componentName:feedName on the feeding component
         retval = []
@@ -820,7 +856,13 @@ class ComponentHeaven(base.ManagerHeaven):
             feeder = set.getFeeder(feederName)
             self.debug('EatersData(): feeder %r' % feeder)
 
+            # what host do we need to connect to, as seen from manager ?
+            # FIXME: this needs to work across every host, not just from
+            # manager
             host = feeder.getListenHost()
+
+            # if the feeder is local, and the eater isn't, then we need to
+            # tell the eater to connect to the manager's host
             if (not self._componentIsLocal(componentAvatar)
                 and host == '127.0.0.1'):
                 host = componentAvatar.getRemoteManagerIP()
@@ -828,30 +870,45 @@ class ComponentHeaven(base.ManagerHeaven):
             retval.append((feederName, host, feeder.getListenPort()))
         return retval
 
-    def _getComponentFeedersData(self, componentAvatar):
+    def _getComponentFeedersData(self, component):
         """
         Retrieves the data of feeders (feed producer elements) for a component.
 
-        @type  componentAvatar: L{flumotion.manager.component.ComponentAvatar}
-        @param componentAvatar: the component
+        @type  component: L{flumotion.manager.component.ComponentAvatar}
+        @param component: the component
 
         @rtype:   tuple of doubles
-        @returns: tuple of (name, host) for each feeder
+        @returns: tuple of (name, host, port) for each feeder
         """
 
-        host = componentAvatar.getClientAddress()
-        feeders = componentAvatar.getFeeders()
-        retval = []
-        for feeder in feeders:
-            retval.append((feeder, host))
-        return retval
+        # get what we think is the IP address where the component is running
+        host = component.getClientAddress()
+        feeders = component.getFeeders()
+        self.debug('returning data for feeders: %r' % (feeders, ))
+        ports = self.vishnu.reservePortsOnWorker(component.getWorkerName(),
+                                                 len(feeders))
+        return map(lambda f, p: (f, host, p), feeders, ports)
 
     def _startComponent(self, componentAvatar):
+        state = componentAvatar.componentState
+        config = state.get('config')
+    
+        # provide master clock if needed
+        if config['clock-master'] == componentAvatar.avatarId:
+            # houston, we have a master clock
+            self.debug('telling component %s to be the clock master' %
+                componentAvatar.avatarId)
+            yield self.provideMasterClock(componentAvatar)
+
+        # start
         eatersData = self._getComponentEatersData(componentAvatar)
         feedersData = self._getComponentFeedersData(componentAvatar)
 
-        componentAvatar.debug('asking component to start with eatersData %s and feedersData %s' % (eatersData, feedersData))
+        componentAvatar.debug(
+            'starting component with eatersData %s and feedersData %s' % (
+                eatersData, feedersData))
         componentAvatar.start(eatersData, feedersData)
+    _startComponent = defer_generator_method(_startComponent)
 
     # FIXME: better name startComponentIfReady
     def checkComponentStart(self, readiness, componentAvatar):
@@ -888,15 +945,61 @@ class ComponentHeaven(base.ManagerHeaven):
         #componentName = componentAvatar.avatarId
         #self.vishnu.adminHeaven.uiStateChanged(componentName, state)
 
+        # ensure it has a parent -- the parent will be null if this is
+        # an already-running worker connects to a freshly restarted
+        # manager
+        state = componentAvatar.componentState
+        if not state:
+            self.warning('Implement manager connection sniffing')
+            return
+
+        if not state.get('parent'):
+            # parent is normally set by the manager when creating a flow
+            # from an xml file. in this case we get a state, and
+            # reconstruct the parent. unfortunately we have to do it by
+            # parsing the avatar id, but hey, life isn't perfect.
+            flowName = componentAvatar.avatarId.split('/')[1]
+            flows = self.vishnu.state.get('flows')
+            try:
+                flow = dict([(x.get('name'),x) for x in flows])[flowName]
+            except KeyError:
+                # FIXME: this is just copied from manager.py
+                self.info('Creating flow "%s"' % flowName)
+                flow = planet.ManagerFlowState()
+                flow.set('name', flowName)
+                flow.set('parent', self.vishnu.state)
+                self.vishnu.state.append('flows', flow)
+            state.set('parent', flow)
+            flow.append('components', state)
+
+        # set up the component so we have feeders and eaters
+        state = componentAvatar.componentState
+        config = state.get('config')
+
+        self.debug('setting up componentAvatar %r' % componentAvatar)
+        d = componentAvatar.setup(config)
+        yield d
+
+        try:
+            d.value()
+        except errors.ComponentSetupError, e:
+            self.warning('Failed to setup component')
+            # FIXME: we cannot start it - is there any reason to go on ?
+            return
+        except Exception, e:
+            self.warning('setup failed:%s' % log.getExceptionMessage(e))
+            raise errors.FlumotionError('Could not set up component')
+
         # tell the feeder set
         set = self._getFeederSet(componentAvatar)
         set.addFeeders(componentAvatar)
 
         # check if we eat feeds from other feeders
-        self.debug('checking if we eat feeds')
+        self.debug('checking if %r eats feeds' % componentAvatar)
         eaterFeeders = componentAvatar.getEaters()
         if not eaterFeeders:
-            componentAvatar.debug('component does not take feeds, starting')
+            componentAvatar.debug(
+                'component does not eat feeds, so starting it right away')
             self._startComponent(componentAvatar)
             self.debug('heaven registered component %r' % componentAvatar)
             return
@@ -910,6 +1013,7 @@ class ComponentHeaven(base.ManagerHeaven):
                 self.checkComponentStart)
 
         self.debug('heaven registered component %r' % componentAvatar)
+    registerComponent = defer_generator_method(registerComponent)
 
     def unregisterComponent(self, componentAvatar):
         """
@@ -924,6 +1028,11 @@ class ComponentHeaven(base.ManagerHeaven):
         set = self._getFeederSet(componentAvatar)
         set.removeFeeders(componentAvatar)
 
+        config = componentAvatar.componentState.get('config')
+        if config['clock-master'] == componentAvatar.avatarId:
+            # houston, we have a master clock
+            self.removeMasterClock(componentAvatar)
+
         # clean up component
         componentAvatar.cleanup()
 
@@ -945,3 +1054,107 @@ class ComponentHeaven(base.ManagerHeaven):
                 feederName, readiness))
         set = self._getFeederSet(componentAvatar)
         set.feederSetReadiness(feederName, readiness)
+
+    def provideMasterClock(self, componentAvatar):
+        """
+        Tell the given component to provide a master clock.
+        Trigger all deferreds waiting on this componentAvatar to provide
+        a master clock.
+        
+        @type componentAvatar: L{ComponentAvatar}
+        
+        @rtype: L{twisted.internet.defer.Deferred}
+        """
+        avatarId = componentAvatar.avatarId
+        self.debug('provideMasterClock on component %s' % avatarId)
+
+        def setMasterClockInfo(result):
+            # we get host, port, base_time
+            # FIXME: host is the default from NetClock, so the local IP,
+            # always.  A little inconvenient.
+            self._masterClockInfo[avatarId] = result
+            return result
+
+        def wakeClockMasterWaiters(result):
+            self.info('Received master clock info from clock master %s' %
+                avatarId)
+            self.debug('got master clock info: %r' % (result, ))
+
+            # wake all components waiting on the clock master info
+            if avatarId in self._clockMasterWaiters:
+                waiters = self._clockMasterWaiters[avatarId]
+                del self._clockMasterWaiters[avatarId]
+                for d, waiterId in waiters:
+                    self.debug(
+                        'giving master clock info to waiting component %s' %
+                        waiterId)
+                    d.callback(result)
+
+        workerName = componentAvatar.getWorkerName()
+        port = self.vishnu.reservePortsOnWorker(workerName, 1)[0]
+
+        if avatarId in self._masterClockInfo:
+            self.warning('component %s already has master clock info: %r'
+                         % (avatarId, self._masterClockInfo[avatarId]))
+            del self._masterClockInfo[avatarId]
+        d = componentAvatar.mindCallRemote('provideMasterClock', port)
+        d.addCallback(setMasterClockInfo)
+        d.addCallback(wakeClockMasterWaiters)
+        return d
+
+    def removeMasterClock(self, componentAvatar):
+        """
+        Tell the given component to stop providing a master clock.
+        
+        @type componentAvatar: L{ComponentAvatar}
+        """
+ 
+        avatarId = componentAvatar.avatarId
+        workerName = componentAvatar.getWorkerName()
+
+        # if any components were waiting on master clock info from this
+        # clock master, errback them
+        if avatarId in self._clockMasterWaiters:
+            waiters = self._clockMasterWaiters[avatarId]
+            del self._clockMasterWaiters[avatarId]
+            for d, waiterId in waiters:
+                self.debug('telling waiting component %s that '
+                    'the clock master %s is gone' % (waiterId, avatarId))
+                d.errback(errors.ComponentStartError(
+                    'clock master component start cancelled'))
+
+        # release our clock port
+        if avatarId in self._masterClockInfo:
+            info = self._masterClockInfo[avatarId]
+            if info:
+                port = info[1]
+                self.vishnu.releasePortsOnWorker(workerName, [port])
+            else:
+                self.debug('avatarId %r has None masterClockInfo' % avatarId)
+            del self._masterClockInfo[avatarId]
+        else:
+            self.warning('component %s has no master clock info'
+                         % (avatarId,))
+
+    def getMasterClockInfo(self, avatarId, waiterId=None):
+        """
+        Get the master clock information from the given clock master component.
+
+        @param avatarId: the id of the clock master.
+        @param waiterId: the id of the requesting component.
+        @returns: a deferred firing an (ip, port, base_time) triple.
+        @rtype:   L{twisted.internet.defer.Deferred}
+        """
+        self.debug('getting master clock info for component %s' % avatarId)
+
+        # if we already have it, return it immediately
+        if avatarId in self._masterClockInfo:
+            return defer.succeed(self._masterClockInfo[avatarId])
+
+        if not avatarId in self._clockMasterWaiters:
+            self._clockMasterWaiters[avatarId] = []
+
+        # otherwise, add a deferred and our own avatarId
+        ret = defer.Deferred()
+        self._clockMasterWaiters[avatarId].append((ret, waiterId))
+        return ret
