@@ -2,7 +2,8 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.
+# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
 # the GNU General Public License version 2 as published by
@@ -26,8 +27,6 @@ API Stability: semi-stable
 
 __all__ = ['ManagerServerFactory', 'Vishnu']
 
-import os
-
 from twisted.internet import reactor, defer
 from twisted.cred import error
 from twisted.python import components, failure
@@ -35,12 +34,35 @@ from twisted.spread import pb
 from twisted.cred import portal
 
 from flumotion.common import bundle, config, errors, interfaces, log, registry
-from flumotion.common import planet, common, dag
+from flumotion.common import planet, common, dag, messages
 from flumotion.common.planet import moods
 from flumotion.configure import configure
 from flumotion.manager import admin, component, worker, base
 from flumotion.twisted import checkers
 from flumotion.twisted import portal as fportal
+from flumotion.twisted.defer import defer_generator_method
+
+from flumotion.common.messages import N_
+T_ = messages.gettexter('flumotion')
+
+def _find(list, value, proc=lambda x: x):
+    return list[[proc(x) for x in list].index(value)]
+
+def _first(list, proc=lambda x: x):
+    for x in list:
+        if proc(x): return x
+
+def _any(list, proc=lambda x: x):
+    return filter(proc, list)
+
+def _fint(*procs):
+    # intersection of functions
+    def int(*args, **kwargs):
+        for p in procs:
+            if not p(*args, **kwargs): return False
+        return True
+    return int
+
 
 # an internal class
 class Dispatcher(log.Loggable):
@@ -72,13 +94,18 @@ class Dispatcher(log.Loggable):
     # to the piece that called login(),
     # which in our case is a component or an admin client.
     def requestAvatar(self, avatarId, mind, *ifaces):
-        avatar = self.createAvatarFor(avatarId, ifaces)
-        self.debug("returning Avatar: id %s, avatar %s" % (avatarId, avatar))
+        try:
+            avatar = self.createAvatarFor(avatarId, ifaces)
+            self.debug("returning Avatar: id %s, avatar %s" % (avatarId, avatar))
+        except errors.AlreadyConnectedError, e:
+            self.debug("component with id %s already logged in" % (avatarId))
+            return defer.fail(failure.Failure(e))
 
         # schedule a perspective attached for after this function
+        # FIXME: there needs to be a way to not have to do a callLater
+        # blindly so cleanup can be guaranteed
         reactor.callLater(0, avatar.attached, mind)
 
-        # return a tuple of interface, aspect, and logout function 
         return (pb.IPerspective, avatar,
                 lambda a=avatar, m=mind, i=avatarId: self.removeAvatar(i, a, m))
 
@@ -162,8 +189,7 @@ class Vishnu(log.Loggable):
         
         self.bouncer = None # used by manager to authenticate worker/component
         
-        self.bundlerBasket = None
-        self._setupBundleBasket()
+        self.bundlerBasket = registry.getRegistry().makeBundlerBasket()
 
         self._componentMappers = {} # any object -> ComponentMapper
 
@@ -203,6 +229,7 @@ class Vishnu(log.Loggable):
         # FIXME: we should be able to create "wanted" config/state from
         # something else than XML as well
         self.configuration = conf = config.FlumotionConfigXML(filename, data)
+        conf.parse()
 
         # scan filename for a bouncer component in the manager
         # FIXME: we should have a "running" state object layout similar
@@ -213,20 +240,28 @@ class Vishnu(log.Loggable):
 
             self.debug('going to start manager bouncer %s of type %s' % (
                 conf.manager.bouncer.name, conf.manager.bouncer.type))
-            from flumotion.common.registry import registry
-            defs = registry.getComponent(conf.manager.bouncer.type)
+            defs = registry.getRegistry().getComponent(
+                conf.manager.bouncer.type)
             configDict = conf.manager.bouncer.getConfigDict()
-            import flumotion.worker.job
-            self.setBouncer(flumotion.worker.job.getComponent(configDict, defs))
+            # FIXME: this is terrible, we shouldn't be importing from
+            # anything outside of flumotion.common and flumotion.manager...
+            from flumotion.job import job
+            try:
+                entry = defs.getEntryByType('component')
+                # FIXME: use entry.getModuleName() (doesn't work atm?)
+                moduleName = defs.getSource()
+                methodName = entry.getFunction()
+            except KeyError:
+                self.warning('no "component" entry in registry of type %s, %s',
+                             type, 'falling back to createComponent')
+                moduleName = defs.getSource()
+                methodName = "createComponent"
+            bouncer = job.createComponent(moduleName, methodName)
+            bouncer.setup(configDict)
+            self.setBouncer(bouncer)
             self.bouncer.debug('started')
             log.info('manager', "Started manager's bouncer")
 
-        # make component heaven
-        # load the configuration as well
-        # FIXME: we should only handle the added conf, so we get the changes
-        # parsing should also be done only once
-        self.componentHeaven.loadConfiguration(filename, data)
-
         # now add stuff from the config that did not exist in self.state yet
         self.debug('syncing up planet state with config')
         added = [] # added components while parsing
@@ -276,11 +311,15 @@ class Vishnu(log.Loggable):
             if not dict.has_key('source'):
                 continue
 
+            # source entries are feederName:
+            # componentName[:feedName], with feedName defaulting to default
             list = dict['source']
 
             # FIXME: there's a bug in config parsing - sometimes this gives us
             # one string, and sometimes a list of one string, and sometimes
             # a list
+            # we make this component depend on every component listed in
+            # source; they have the same parent by design
             if isinstance(list, str):
                 list = [list, ]
             for eater in list:
@@ -288,7 +327,7 @@ class Vishnu(log.Loggable):
                 if ':' in name:
                     name = eater.split(':')[0]
 
-                flowName = state.get('parent').get('name')
+                flowName = state.get('parent').get('name')  
                 avatarId = common.componentPath(name, flowName)
                 parentState = self._componentMappers[avatarId].state
                 self.debug('depending %r on %r' % (state, parentState))
@@ -306,11 +345,11 @@ class Vishnu(log.Loggable):
                 self.debug('no components scheduled for worker %s' % workerId)
             else:
                 avatar = self.workerHeaven.avatars[workerId]
-                self._workerStartComponents(avatar, ours)
-            # make sure components with no worker specified don't get started
-            # on all different workers
-            for c in ours:
-                components.remove(c)
+                self._workerCreateComponents(avatar, ours)
+                # they're started, so remove them from the running list of
+                # components
+                for c in ours:
+                    components.remove(c)
  
     def _addComponent(self, config, parent):
         """
@@ -331,8 +370,7 @@ class Vishnu(log.Loggable):
         state.set('parent', parent)
         parent.append('components', state)
 
-        parentName = parent.get('name')
-        avatarId = common.componentPath(config.name, parentName)
+        avatarId = config.getConfigDict()['avatarId']
 
         # add to mapper
         m = ComponentMapper()
@@ -346,26 +384,6 @@ class Vishnu(log.Loggable):
 
         return state
 
-    def _setupBundleBasket(self):
-        self.bundlerBasket = bundle.BundlerBasket()
-
-        for b in registry.registry.getBundles():
-            bundleName = b.getName()
-            self.debug('Adding bundle %s' % bundleName)
-            for d in b.getDirectories():
-                directory = d.getName()
-                for filename in d.getFiles():
-                    fullpath = os.path.join(configure.pythondir, directory,
-                                            filename.getLocation())
-                    relative = filename.getRelative()
-                    self.log('Adding path %s as %s to bundle %s' % (
-                        fullpath, relative, bundleName))
-                    self.bundlerBasket.add(bundleName, fullpath, relative)
-
-            for d in b.getDependencies():
-                self.log('Adding dependency of %s on %s' % (bundleName, d))
-                self.bundlerBasket.depend(bundleName, d)
-        
     def _createHeaven(self, interface, klass):
         """
         Create a heaven of the given klass that will send avatars to clients
@@ -391,9 +409,10 @@ class Vishnu(log.Loggable):
     def getFactory(self):
         return self.factory
        
-    def componentStart(self, componentState):
+    def componentCreate(self, componentState):
         """
-        Start the given component.
+        Create the given component.  This will currently also trigger
+        a start eventually when the component avatar attaches.
 
         The component should be sleeping.
         The worker it should be started on should be present.
@@ -437,9 +456,25 @@ class Vishnu(log.Loggable):
             raise errors.ComponentNoWorkerError(
                 "Could not find any worker to start on")
 
-        # now that we have a worker, get started
-        return self._workerStartComponents(worker, [componentState])
+        # now that we have a worker, get going
+        return self._workerCreateComponents(worker, [componentState])
 
+    def componentAddMessage(self, avatarId, message):
+        """
+        Set the given message on the given component's state.
+        Can be called e.g. by a worker to report on a crashed component.
+        """
+        if not avatarId in self._componentMappers:
+            self.warning('asked to set a message on non-mapped component %s' %
+                avatarId)
+            return
+
+        m = self._componentMappers[avatarId]
+        m.state.append('messages', message)
+        if message.level == messages.ERROR:
+            self.debug('Error message makes component sad')
+            m.state.set('mood', moods.sad.value)
+        
     # FIXME: unify naming of stuff like this
     def workerAttached(self, workerAvatar):
         # called when a worker logs in
@@ -462,18 +497,19 @@ class Vishnu(log.Loggable):
             self.debug('vishnu.workerAttached(): no components for this worker')
             return
 
-        self._workerStartComponents(workerAvatar, components)
+        self._workerCreateComponents(workerAvatar, components)
             
-    def _workerStartComponents(self, workerAvatar, components):
+    def _workerCreateComponents(self, workerAvatar, components):
         """
-        Start the list of components on the given worker, sequentially.
+        Create the list of components on the given worker, sequentially, but
+        in no specific order.
 
         @type  workerAvatar: L{flumotion.manager.worker.WorkerAvatar}
         @type  components:   list of
                              L{flumotion.common.planet.ManagerComponentState}
         """
         names = [c.get('name') for c in components]
-        self.debug('starting components %r' % names)
+        self.debug('creating components %r' % names)
         
         d = defer.Deferred()
 
@@ -482,43 +518,55 @@ class Vishnu(log.Loggable):
             parentName = c.get('parent').get('name')
             type = c.get('type')
             config = c.get('config')
-            self.debug('_workerStartComponents(): scheduling start of /%s/%s on %s' % (
+            self.debug(
+                '_workerCreateComponents(): scheduling create of /%s/%s on %s' % (
                 parentName, componentName, workerAvatar.avatarId))
             
-            d.addCallback(self._workerStartComponentDelayed,
+            d.addCallback(self._workerCreateComponentDelayed,
                 workerAvatar, c, type, config)
 
         d.addCallback(lambda result: self.debug(
-            '_workerStartComponents(): completed start chain'))
+            '_workerCreateComponents(): completed setting up create chain'))
 
         # now trigger the chain
-        self.debug('_workerStartComponents(): triggering start chain')
+        self.debug('_workerCreateComponents(): triggering create chain')
         d.callback(None)
         #reactor.callLater(0, d.callback, None)
         return d
 
-    def _workerStartComponentDelayed(self, result, workerAvatar,
+    def _workerCreateComponentDelayed(self, result, workerAvatar,
             componentState, type, config):
 
-        m = self._componentMappers[componentState]
-        avatarId = m.id
+        avatarId = config['avatarId']
 
-        # FIXME: rename to startComp
-        d = workerAvatar.start(avatarId, type, config)
+        d = workerAvatar.createComponent(avatarId, type, config)
         # FIXME: here we get the avatar Id of the component we wanted
         # started, so now attach it to the planetState's component state
-        d.addCallback(self._startCallback, componentState)
-        d.addErrback(self._startErrback, componentState)
+        d.addCallback(self._createCallback, componentState)
+        d.addErrback(self._createErrback, componentState)
 
-    def _startCallback(self, result, componentState):
+        # FIXME: shouldn't we return d here to make sure components
+        # wait on each other to be started ?
+
+    def _createCallback(self, result, componentState):
         self.debug('got avatarId %s for state %s' % (result, componentState))
         m = self._componentMappers[componentState]
         assert result == m.id, "received id %s is not the expected id %s" % (
             result, m.id)
 
-    def _startErrback(self, error, state):
-        self.info('failed to start component %s: %s'
-                  % (state.get('name'), error.getErrorMessage()))
+    def _createErrback(self, failure, state):
+        # FIXME: make ConfigError copyable so we can .check() it here
+        # and print a nicer warning
+        self.warning('failed to create component %s: %s'
+                  % (state.get('name'), failure.getErrorMessage()))
+
+        message = messages.Error(T_(
+            N_("The component could not be started.")),
+                debug=failure.getErrorMessage())
+
+        state.set('mood', moods.sad.value)
+        state.append('messages', message)
+
         return None
 
     def workerDetached(self, workerAvatar):
@@ -526,18 +574,56 @@ class Vishnu(log.Loggable):
         workerId = workerAvatar.avatarId
         self.debug('vishnu.workerDetached(): id %s' % workerId)
 
+    def _configToComponentState(self, config, avatar):
+        assert not avatar.avatarId in self._componentMappers.keys()
+
+        state = planet.ManagerComponentState()
+        state.set('name', config['name'])
+        state.set('type', config['type'])
+        state.set('workerRequested', None)
+        state.set('mood', moods.waking.value)
+        state.set('config', config)
+
+        # check if we have this flow yet and add if not
+        isOurFlow = lambda x: x.get('name') == config['parent']
+        flow = _first(self.state.get('flows'), isOurFlow)
+        if not flow:
+            self.info('Creating flow "%s"' % config['parent'])
+            flow = planet.ManagerFlowState()
+            flow.set('name', config['parent'])
+            flow.set('parent', self.state)
+            self.state.append('flows', flow)
+
+        state.set('parent', flow)
+        flow.append('components', state)
+
+        # add to mapper
+        m = ComponentMapper()
+        m.state = state
+        m.id = avatar.avatarId
+        self._componentMappers[m.state] = m
+        self._componentMappers[m.id] = m
+
+        # add nodes to graph
+        self._dag.addNode(state)
+
+        return self.componentAttached(avatar)
+
     def componentAttached(self, componentAvatar):
         # called when a component logs in and gets a component avatar created
         id = componentAvatar.avatarId
         if not id in self._componentMappers.keys():
-            self.warning('id %s not found' % id)
-            return
+            # the manager quit and the component is logging back in
+            d = componentAvatar.mindCallRemote('getConfig')
+            d.addCallback(self._configToComponentState, componentAvatar)
+            return d
         m = self._componentMappers[id]
         m.avatar = componentAvatar
         self._componentMappers[componentAvatar] = m
 
         # attach componentstate to avatar
         componentAvatar.componentState = m.state
+        return defer.succeed(None)
 
     def componentDetached(self, componentAvatar):
         # called when the component has detached
@@ -549,10 +635,6 @@ class Vishnu(log.Loggable):
         # called when the jobstate is retrieved
         self.debug('vishnu registering component %r' % componentAvatar)
 
-        #if not componentAvatar in self._componentMappers.keys():
-        #    self.warning('avatar %r not found' % componentAvatar)
-        #    return
-
         # map jobState
         jobState = componentAvatar.jobState
         m = self._componentMappers[componentAvatar]
@@ -579,11 +661,13 @@ class Vishnu(log.Loggable):
             self.warning('Could not remove jobState for %r' % componentAvatar)
         m.jobState = None
         
-        m.state.set('mood', moods.sleeping.value)
+        # if the component was sad, keep it sad.  It still needs manual
+        # admin intervention.
+        if m.state.get('mood') != moods.sad.value:   
+            m.state.set('mood', moods.sleeping.value)
         m.state.set('pid', None)
+        m.state.set('cpu', None)
         m.state.set('workerName', None)
-        m.state.set('message', 'Component "%s" logged out' %
-            m.state.get('name'))
 
         # unmap avatar
         del self._componentMappers[m.avatar]
@@ -600,6 +684,31 @@ class Vishnu(log.Loggable):
 
         return list
 
+    def deleteFlow(self, flowName):
+        """
+        Empty the planet of all components, and flows.
+
+        @returns: a deferred that will fire when the planet is empty.
+        """
+
+        # first get all components to sleep
+        flow = _find(self.state.get('flows'), flowName, lambda x: x.get('name'))
+        components = flow.get('components')
+
+        # if any component is already in a mood change/command, fail
+        isBusy = lambda c: c.get('moodPending') != None
+        isNotSleeping = lambda c: c.get('mood') is not moods.sleeping.value
+        pred = _fint(isBusy, isNotSleeping)
+        if _any(components, pred):
+            raise errors.BusyComponentError(_first(components, pred))
+
+        for c in components:
+            del self._componentMappers[self._componentMappers[c].id]
+            del self._componentMappers[c]
+        yield flow.empty()
+        yield self.state.remove('flows', flow)
+    deleteFlow = defer_generator_method(deleteFlow)
+        
     def emptyPlanet(self):
         """
         Empty the planet of all components, and flows.
@@ -684,7 +793,32 @@ class Vishnu(log.Loggable):
 
         return components
 
- 
+    def reservePortsOnWorker(self, workerName, numPorts):
+        """
+        Requests a number of ports on the worker named workerName. The
+        ports will be reserved for the use of the caller until
+        releasePortsOnWorker is called.
+
+        @returns: a list of ports as integers
+        """
+        if not workerName in self.workerHeaven.avatars:
+            raise errors.ComponentNoWorkerError("Worker %s not logged in?"
+                                                % workerName)
+
+        return self.workerHeaven.avatars[workerName].reservePorts(numPorts)
+        
+    def releasePortsOnWorker(self, workerName, ports):
+        """
+        Tells the manager that the given ports are no longer being used,
+        and may be returned to the allocation pool.
+        """
+        if not workerName in self.workerHeaven.avatars:
+            self.warning("Worker %s appears to have logged out, not releasing"
+                       % workerName)
+            return
+
+        return self.workerHeaven.avatars[workerName].releasePorts(ports)
+        
     def getComponentMapper(self, object):
         """
         Look up an object mapper given the object.
